@@ -2,19 +2,17 @@
 #define __CLASS_MYSQL_SESSION_H
 #include "proxysql.h"
 #include "cpp.h"
-/*
-class MySQL_Session_userinfo {
-	public:
-  char *username;
-  char *password;
-  char *schemaname;
-	MySQL_Session_userinfo();
-	~MySQL_Session_userinfo();
-	void set(char *, char *, char *);
-	void set(MySQL_Session_userinfo *);
-	bool set_schemaname(char *, int);
+
+enum proxysql_session_type {
+	PROXYSQL_SESSION_MYSQL,
+	PROXYSQL_SESSION_ADMIN,
+	PROXYSQL_SESSION_STATS,
+	PROXYSQL_SESSION_SQLITE,
+	PROXYSQL_SESSION_CLICKHOUSE,
+	PROXYSQL_SESSION_MYSQL_EMU,
+
+	PROXYSQL_SESSION_NONE
 };
-*/
 
 // these structs will be used for various regex hardcoded
 // their initial use will be for sql_log_bin , sql_mode and time_zone
@@ -40,7 +38,11 @@ class Query_Info {
 
 	MYSQL_STMT *mysql_stmt;
 	stmt_execute_metadata_t *stmt_meta;
+#ifndef PROXYSQL_STMT_V14
 	uint32_t stmt_global_id;
+#else
+	uint64_t stmt_global_id;
+#endif
 	MySQL_STMT_Global_info *stmt_info;
 
 	int QueryLength;
@@ -100,8 +102,8 @@ class MySQL_Session
 	bool handler_again___verify_backend_autocommit();
 	bool handler_again___verify_backend_user_schema();
 	bool handler_again___verify_backend_sql_log_bin();
- 	bool handler_again___verify_backend_sql_mode();
- 	bool handler_again___verify_backend_time_zone();
+	bool handler_again___verify_backend_sql_mode();
+	bool handler_again___verify_backend_time_zone();
 	bool handler_again___status_SETTING_INIT_CONNECT(int *);
 	bool handler_again___status_SETTING_SQL_LOG_BIN(int *);
 	bool handler_again___status_SETTING_SQL_MODE(int *);
@@ -111,18 +113,13 @@ class MySQL_Session
 	bool handler_again___status_CHANGING_USER_SERVER(int *);
 	bool handler_again___status_CHANGING_CHARSET(int *);
 	bool handler_again___status_CHANGING_AUTOCOMMIT(int *);
-
-
-//	void return_MySQL_Connection_To_Poll(MySQL_Data_Stream *);
-
-
-//	MySQL_STMT_Manager *Session_STMT_Manager;
+	void init();
+	void reset();
 
 	//this pointer is always initialized inside handler().
 	// it is an attempt to start simplifying the complexing of handler()
 	PtrSize_t *pktH;
 
-	Session_Regex **match_regexes;
 
 	public:
 	void * operator new(size_t);
@@ -148,12 +145,13 @@ class MySQL_Session
 	char * default_schema;
 
 	uint32_t thread_session_id;
-	unsigned int last_insert_id;
+	unsigned long long last_insert_id;
 	enum session_status status;
 	int healthy;
 	int user_max_connections;
 	int current_hostgroup;
 	int default_hostgroup;
+	int next_query_flagIN;
 	int mirror_hostgroup;
 	int mirror_flagOUT;
 	int active_transactions;
@@ -161,16 +159,17 @@ class MySQL_Session
 	int transaction_persistent_hostgroup;
 	int to_process;
 	int pending_connect;
+	enum proxysql_session_type session_type;
 
 	// bool
 	bool autocommit;
 	bool killed;
-	bool admin;
+	//bool admin;
 	bool max_connections_reached;
 	bool client_authenticated;
 	bool connections_handler;
 	bool mirror;
-	bool stats;
+	//bool stats;
 	bool schema_locked;
 	bool transaction_persistent;
 	bool session_fast_forward;
@@ -179,8 +178,9 @@ class MySQL_Session
 	MySQL_STMTs_meta *sess_STMTs_meta;
 	StmtLongDataHandler *SLDH;
 
+	Session_Regex **match_regexes;
+
 	MySQL_Session();
-//	MySQL_Session(int);
 	~MySQL_Session();
 
 	void set_unhealthy();
@@ -197,11 +197,9 @@ class MySQL_Session
 		}
 		status=e;
 	}
-	//MySQL_Protocol myprot_client;
-	//MySQL_Protocol myprot_server;
 	int handler();
 
-	void (*admin_func) (MySQL_Session *arg, ProxySQL_Admin *, PtrSize_t *pkt);
+	void (*handler_function) (MySQL_Session *arg, void *, PtrSize_t *pkt);
 	MySQL_Backend * find_backend(int);
 	MySQL_Backend * create_backend(int, MySQL_Data_Stream *_myds=NULL);
 	MySQL_Backend * find_or_create_backend(int, MySQL_Data_Stream *_myds=NULL);
@@ -210,6 +208,7 @@ class MySQL_Session
 	void MySQL_Result_to_MySQL_wire(MYSQL *mysql, MySQL_ResultSet *MyRS, MySQL_Data_Stream *_myds=NULL);
 	void MySQL_Stmt_Result_to_MySQL_wire(MYSQL_STMT *stmt, MySQL_Connection *myconn);
 	unsigned int NumActiveTransactions();
+	bool HasOfflineBackends();
 	int FindOneActiveTransaction();
 	unsigned long long IdleTime();
 
