@@ -3,7 +3,9 @@
 #define ____CLASS_STANDARD_MYSQL_THREAD_H
 #include "proxysql.h"
 #include "cpp.h"
+#ifdef IDLE_THREADS
 #include <sys/epoll.h>
+#endif // IDLE_THREADS
 
 #define MIN_POLL_LEN 8
 #define MIN_POLL_DELETE_RATIO  8
@@ -11,10 +13,13 @@
 
 #define ADMIN_HOSTGROUP	-2
 #define STATS_HOSTGROUP	-3
+#define SQLITE_HOSTGROUP -4
 
-#define MYSQL_DEFAULT_SQL_MODE ""
-#define MYSQL_DEFAULT_TIME_ZONE "SYSTEM"
 
+#define MYSQL_DEFAULT_SQL_MODE	""
+#define MYSQL_DEFAULT_TIME_ZONE	"SYSTEM"
+
+#define PROXYSQL_MYSQL_PTHREAD_MUTEX
 
 static unsigned int near_pow_2 (unsigned int n) {
   unsigned int i = 1;
@@ -22,12 +27,14 @@ static unsigned int near_pow_2 (unsigned int n) {
   return i ? i : n;
 }
 
+#ifdef IDLE_THREADS
 typedef struct __attribute__((aligned(CACHE_LINE_SIZE))) _conn_exchange_t {
 	pthread_mutex_t mutex_idles;
 	PtrArray *idle_mysql_sessions;
 	pthread_mutex_t mutex_resumes;
 	PtrArray *resume_mysql_sessions;
 } conn_exchange_t;
+#endif // IDLE_THREADS
 
 class ProxySQL_Poll {
 
@@ -65,7 +72,11 @@ class ProxySQL_Poll {
 	volatile int pending_listener_del;
 
   ProxySQL_Poll() {
+#ifdef PROXYSQL_STATSCOUNTERS_NOLOCK
+		loop_counters=new StatCounters(15,10);
+#else
 		loop_counters=new StatCounters(15,10,false);
+#endif
 		poll_timeout=0;
 		loops=0;
 		len=0;
@@ -94,8 +105,6 @@ class ProxySQL_Poll {
 		delete loop_counters;
   };
 
-
-
   void add(uint32_t _events, int _fd, MySQL_Data_Stream *_myds, unsigned long long sent_time) {
     if (len==size) {
       expand(1);
@@ -146,8 +155,9 @@ class ProxySQL_Poll {
 
 class MySQL_Thread
 {
-
 	private:
+	unsigned int servers_table_version_previous;
+	unsigned int servers_table_version_current;
   unsigned long long last_processing_idles;
 	MySQL_Connection **my_idle_conns;
   bool processing_idles;
@@ -155,26 +165,36 @@ class MySQL_Thread
 
 	PtrArray *cached_connections;
 
+#ifdef IDLE_THREADS
 	struct epoll_event events[MY_EPOLL_THREAD_MAXEVENTS];
 	int efd;
 	unsigned int mysess_idx;
 	std::map<unsigned int, unsigned int> sessmap;
+#endif // IDLE_THREADS
+
+	Session_Regex **match_regexes;
 
 	protected:
 	int nfds;
 
 	public:
 
+	void *gen_args;	// this is a generic pointer to create any sort of structure
+
 	ProxySQL_Poll mypolls;
 	pthread_t thread_id;
 	unsigned long long curtime;
 	unsigned long long pre_poll_time;
 	unsigned long long last_maintenance_time;
 	PtrArray *mysql_sessions;
+	PtrArray *mirror_queue_mysql_sessions;
+	PtrArray *mirror_queue_mysql_sessions_cache;
+#ifdef IDLE_THREADS
 	PtrArray *idle_mysql_sessions;
 	PtrArray *resume_mysql_sessions;
 
 	conn_exchange_t myexchange;
+#endif // IDLE_THREADS
 
 	int pipefd[2];
 	int shutdown;
@@ -186,9 +206,12 @@ class MySQL_Thread
 	// in this way, there is no need for atomic operation and there is no cache miss
 	// when it is needed a total, all threads are checked
 	struct {
-		unsigned long long stmt_prepare;
-		unsigned long long stmt_execute;
-		unsigned long long stmt_close;
+		unsigned long long backend_stmt_prepare;
+		unsigned long long backend_stmt_execute;
+		unsigned long long backend_stmt_close;
+		unsigned long long frontend_stmt_prepare;
+		unsigned long long frontend_stmt_execute;
+		unsigned long long frontend_stmt_close;
 		unsigned long long queries;
 		unsigned long long queries_slow;
 		unsigned long long queries_backends_bytes_sent;
@@ -205,7 +228,11 @@ class MySQL_Thread
 	} status_variables;
 
 
+#ifdef PROXYSQL_MYSQL_PTHREAD_MUTEX
+  pthread_mutex_t thread_mutex;
+#else
   rwlock_t thread_mutex;
+#endif
   MySQL_Thread();
   ~MySQL_Thread();
   MySQL_Session * create_new_session_and_client_data_stream(int _fd);
@@ -221,9 +248,6 @@ class MySQL_Thread
   void refresh_variables();
   void register_session_connection_handler(MySQL_Session *_sess, bool _new=false);
   void unregister_session_connection_handler(int idx, bool _new=false);
-  //void myds_backend_set_failed_connect(MySQL_Data_Stream *myds, unsigned int n);
-  //void myds_backend_pause_connect(MySQL_Data_Stream *myds);
-  //void myds_backend_first_packet_after_connect(MySQL_Data_Stream *myds, unsigned int n);
   void listener_handle_new_connection(MySQL_Data_Stream *myds, unsigned int n);
 	void Get_Memory_Stats();
 	MySQL_Connection * get_MyConn_local(unsigned int);
@@ -232,11 +256,9 @@ class MySQL_Thread
 };
 
 
-
 typedef MySQL_Thread * create_MySQL_Thread_t();
 typedef void destroy_MySQL_Thread_t(MySQL_Thread *);
 
-
 class iface_info {
 	public:
 	char *iface;
@@ -256,9 +278,6 @@ class iface_info {
 	}
 };
 
-
-
-
 class MySQL_Listeners_Manager {
 	private:
 	PtrArray *ifaces;
@@ -266,7 +285,6 @@ class MySQL_Listeners_Manager {
   MySQL_Listeners_Manager();
 	~MySQL_Listeners_Manager();
 	int add(const char *iface, unsigned int num_threads, int **perthrsocks);
-	//int add(const char *address, int port);
 	int find_idx(const char *iface);
 	int find_idx(const char *address, int port);
 	iface_info * find_iface_from_fd(int fd);
@@ -274,15 +292,20 @@ class MySQL_Listeners_Manager {
 	void del(unsigned int idx);
 };
 
-
-
 class MySQL_Threads_Handler
 {
 	private:
 	int shutdown_;
 	size_t stacksize;
 	pthread_attr_t attr;
+#ifdef PROXYSQL_MYSQL_PTHREAD_MUTEX
+	pthread_rwlock_t rwlock;
+#else
 	rwlock_t rwlock;
+#endif
+	PtrArray *bind_fds;
+	MySQL_Listeners_Manager *MLM;
+	public:
 	struct {
 		int monitor_history;
 		int monitor_connect_interval;
@@ -293,9 +316,12 @@ class MySQL_Threads_Handler
 		int monitor_read_only_interval;
 		int monitor_read_only_timeout;
 		bool monitor_enabled;
+		bool monitor_wait_timeout;
 		bool monitor_writer_is_also_reader;
 		int monitor_replication_lag_interval;
 		int monitor_replication_lag_timeout;
+		int monitor_groupreplication_healthcheck_interval;
+		int monitor_groupreplication_healthcheck_timeout;
 		int monitor_query_interval;
 		int monitor_query_timeout;
 		int monitor_slave_lag_when_null;
@@ -308,12 +334,15 @@ class MySQL_Threads_Handler
 		int query_retries_on_failure;
 		int connect_retries_on_failure;
 		int connect_retries_delay;
+		int connection_delay_multiplex_ms;
 		int connection_max_age_ms;
 		int connect_timeout_server;
 		int connect_timeout_server_max;
 		int free_connections_pct;
+#ifdef IDLE_THREADS
 		int session_idle_ms;
 		bool session_idle_show_processlist;
+#endif // IDLE_THREADS
 		bool sessions_sort;
 		char *default_schema;
 		char *interfaces;
@@ -328,7 +357,9 @@ class MySQL_Threads_Handler
 		bool client_found_rows;
 		bool multiplexing;
 //		bool stmt_multiplexing;
+		bool forward_autocommit;
 		bool enforce_autocommit_on_reads;
+		bool autocommit_false_not_reusable;
 		int max_allowed_packet;
 		int max_transaction_time;
 		int threshold_query_length;
@@ -339,11 +370,15 @@ class MySQL_Threads_Handler
 		int max_connections;
 		int max_stmts_per_connection;
 		int max_stmts_cache;
+		int mirror_max_concurrency;
+		int mirror_max_queue_length;
 		int default_max_latency_ms;
 		int default_query_delay;
 		int default_query_timeout;
 		int query_processor_iterations;
+		int query_processor_regex;
 		int long_query_time;
+		int hostgroup_manager_verbose;
 		char *init_connect;
 		char *default_sql_mode;
 		char *default_time_zone;
@@ -362,17 +397,14 @@ class MySQL_Threads_Handler
 		char * ssl_p2s_cipher;
 		int query_cache_size_MB;
 	} variables;
-	PtrArray *bind_fds;
-	MySQL_Listeners_Manager *MLM;
-	public:
+	struct {
+		unsigned int mirror_sessions_current;
+	} status_variables;
 	unsigned int num_threads;
 	proxysql_mysql_thread_t *mysql_threads;
+#ifdef IDLE_THREADS
 	proxysql_mysql_thread_t *mysql_threads_idles;
-	//rwlock_t rwlock_idles;
-	//rwlock_t rwlock_resumes;
-	//PtrArray *idle_mysql_sessions;
-	//PtrArray *resume_mysql_sessions;
-	//virtual const char *version() {return NULL;};
+#endif // IDLE_THREADS
 	unsigned int get_global_version();
 	void wrlock();
  	void wrunlock();
@@ -403,15 +435,21 @@ class MySQL_Threads_Handler
 	SQLite3_result * SQL3_Processlist();
 	SQLite3_result * SQL3_GlobalStatus();
 	bool kill_session(uint32_t _thread_session_id);
-	unsigned long long get_total_stmt_prepare();
-	unsigned long long get_total_stmt_execute();
-	unsigned long long get_total_stmt_close();
+	unsigned long long get_total_mirror_queue();
+	unsigned long long get_total_backend_stmt_prepare();
+	unsigned long long get_total_backend_stmt_execute();
+	unsigned long long get_total_backend_stmt_close();
+	unsigned long long get_total_frontend_stmt_prepare();
+	unsigned long long get_total_frontend_stmt_execute();
+	unsigned long long get_total_frontend_stmt_close();
 	unsigned long long get_total_queries();
 	unsigned long long get_slow_queries();
 	unsigned long long get_queries_backends_bytes_recv();
 	unsigned long long get_queries_backends_bytes_sent();
 	unsigned int get_active_transations();
+#ifdef IDLE_THREADS
 	unsigned int get_non_idle_client_connections();
+#endif // IDLE_THREADS
 	unsigned long long get_query_processor_time();
 	unsigned long long get_backend_query_time();
 	unsigned long long get_mysql_backend_buffers_bytes();
