@@ -1,4 +1,3 @@
-//#ifdef __cplusplus
 #include "gen_utils.h"
 #include "MySQL_Thread.h"
 #include "MySQL_Session.h"
@@ -7,32 +6,28 @@
 #include "query_cache.hpp"
 #include "mysql_connection.h"
 #include "sqlite3db.h"
-//#include "simple_kv.h"
 #include "StatCounters.h"
 #include "MySQL_Monitor.hpp"
 #include "MySQL_Protocol.h"
 #include "MySQL_Authentication.hpp"
+#ifdef PROXYSQLCLICKHOUSE
+#include "ClickHouse_Authentication.hpp"
+#endif /* PROXYSQLCLICKHOUSE */
 #include "fileutils.hpp"
 #include "configfile.hpp"
 #include "query_processor.h"
 #include "proxysql_admin.h"
+#include "SQLite3_Server.h"
+#ifdef PROXYSQLCLICKHOUSE
+#include "ClickHouse_Server.h"
+#endif /* PROXYSQLCLICKHOUSE */
 #include "MySQL_HostGroups_Manager.h"
 #include "MySQL_Logger.hpp"
 #include "MySQL_PreparedStatement.h"
+#include "ProxySQL_Cluster.hpp" // cluster
 #undef swap
 #undef min
 #undef max
-//#include <memory>
 #include <stdio.h>
-//#include <algorithm>
-//#include <functional>
-//#include <type_traits>
-//#include <iosfwd>
 #include <map>
 #include <unordered_map>
-//#include <set>
-//#include <sstream>
-//#include <string>
-//#include <utility>
-//#include <vector>
-//#endif /* __cplusplus */
