@@ -11,8 +11,9 @@
 #define STATUS_MYSQL_CONNECTION_LOCK_TABLES          0x00000010
 #define STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE      0x00000020
 #define STATUS_MYSQL_CONNECTION_GET_LOCK             0x00000040
+#define STATUS_MYSQL_CONNECTION_NO_MULTIPLEX         0x00000080
 #define STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0         0x00000100
-#define STATUS_MYSQL_CONNECTION_FOUND_ROWS           0x00000080
+#define STATUS_MYSQL_CONNECTION_FOUND_ROWS           0x00000200
 
 class MySQL_Connection_userinfo {
 	private:
@@ -30,8 +31,6 @@ class MySQL_Connection_userinfo {
 	bool set_schemaname(char *, int);
 };
 
-
-
 class MySQL_Connection {
 	private:
 	bool is_expired(unsigned long long timeout);
@@ -50,8 +49,8 @@ class MySQL_Connection {
 		bool init_connect_sent;
 		uint8_t protocol_version;
 		uint8_t charset;
-		int8_t last_set_autocommit;
 		uint8_t sql_log_bin;
+		int8_t last_set_autocommit;
 		bool autocommit;
 	} options;
 	struct {
@@ -66,7 +65,11 @@ class MySQL_Connection {
 	unsigned long long last_time_used;
 	unsigned long long timeout;
 	int fd;
+#ifndef PROXYSQL_STMT_V14
 	MySQL_STMTs_local *local_stmts;	// local view of prepared statements
+#else
+	MySQL_STMTs_local_v14 *local_stmts;	// local view of prepared statements
+#endif
 	MYSQL *mysql;
 	MYSQL *ret_mysql;
 	MYSQL_RES *mysql_result;
@@ -91,15 +94,12 @@ class MySQL_Connection {
 	bool processing_prepared_statement_prepare;
 	bool processing_prepared_statement_execute;
 	bool processing_multi_statement;
+	bool multiplex_delayed;
 	MySQL_Connection();
 	~MySQL_Connection();
-//	int assign_mshge(unsigned int);
-	//void set_mshge(MySQL_Hostgroup_Entry *);
-//	void free_mshge();
 	bool set_autocommit(bool);
 	uint8_t set_charset(uint8_t);
 
-	void set_status_sql_log_bin0(bool);
 	void set_status_transaction(bool);
 	void set_status_compression(bool);
 	void set_status_get_lock(bool);
@@ -107,6 +107,8 @@ class MySQL_Connection {
 	void set_status_temporary_table(bool);
 	void set_status_prepared_statement(bool);
 	void set_status_user_variable(bool);
+	void set_status_no_multiplex(bool);
+	void set_status_sql_log_bin0(bool);
 	void set_status_found_rows(bool);
 	bool get_status_transaction();
 	bool get_status_compression();
@@ -115,6 +117,7 @@ class MySQL_Connection {
 	bool get_status_temporary_table();
 	bool get_status_prepared_statement();
 	bool get_status_user_variable();
+	bool get_status_no_multiplex();
 	bool get_status_sql_log_bin0();
 	bool get_status_found_rows();
 	void connect_start();
@@ -155,7 +158,14 @@ class MySQL_Connection {
 
 
 	void async_free_result();
-	bool IsActiveTransaction();
+	bool IsActiveTransaction() {
+		bool ret=false;
+			if (mysql) {
+				ret = (mysql->server_status & SERVER_STATUS_IN_TRANS);
+			}
+		return ret;
+	}
+	bool IsServerOffline();
 	bool IsAutoCommit();
 	bool MultiplexDisabled();
 	void ProcessQueryAndSetStatusFlags(char *query_digest_text);
@@ -164,5 +174,6 @@ class MySQL_Connection {
 
 	void set_is_client(); // used for local_stmts
 
+	void reset();
 };
 #endif /* __CLASS_MYSQL_CONNECTION_H */
