@@ -3,15 +3,8 @@
 #include <stack>
 
 #include <algorithm>
-//#include <functional>
-//#include <memory>
-//#include <string>
-//#include <utility>
 
-
-//#include "btree_map.h"
 #ifndef EZOPTION
-//#include "ezOptionParser.hpp"
 #define EZOPTION
 #endif /* EZOPTION */
 #endif
@@ -28,7 +21,6 @@
 #include <assert.h>
 #include <unistd.h>
 #include <sys/socket.h>
-//#include <resolv.h>
 #include <arpa/inet.h>
 #include <pthread.h>
 #include <sys/wait.h>
@@ -50,28 +42,22 @@
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <poll.h>
-//#include <glib.h>
 #include <execinfo.h>
 
 #include <dlfcn.h>
 
-
-//#include <event2/buffer.h>
-//#include <event2/thread.h>
-
-//#include <sys/epoll.h>
 #include <sys/ioctl.h>
 
-
+#if !defined(__FreeBSD__) && !defined(__APPLE__)
 #define HAVE_BOOL
 #include "my_global.h"
-#include "my_pthread.h"
+//#include "my_pthread.h"
+#endif
 #include "mysql.h"
 #include "mysql_com.h"
 
 #include "proxysql_mem.h"
 
-
 #include "proxysql_structs.h"
 #include "proxysql_debug.h"
 #include "proxysql_macros.h"
@@ -91,15 +77,12 @@
 
 #include "c_tokenizer.h"
 
-//#include "cpp.h"
-
-
 #ifdef DEBUG
 #define DEB "_DEBUG"
 #else
 #define DEB ""
 #endif /* DEBUG */
-#define PROXYSQL_VERSION	"1.3.7-1.1" DEB
+#define PROXYSQL_VERSION	"v1.4.3-1.1" DEB
 #define PROXYSQL_CODENAME	"Truls"
 
 #ifndef PROXYSQL_FUNC_DEFS
@@ -107,15 +90,11 @@
 
 #ifdef __cplusplus
 extern "C" {
-//int parse_mysql_pkt(unsigned char *, MySQL_Data_Stream *, int);
 #endif /* __cplusplus */
 
-//mysql_data_stream_t * mysql_data_stream_New(mysql_session_t *, int, mysql_backend_t *);
 int listen_on_port(char *ip, uint16_t port, int backlog, bool reuseport=false);
 int listen_on_unix(char *, int);
 int connect_socket(char *, int);
-//void process_global_variables_from_file(GKeyFile *, int );
-//void main_opts(gint *, gchar ***);
 int config_file_is_readable(char *);
 unsigned int CPY3(unsigned char *);
 
@@ -124,9 +103,6 @@ int pkt_end(unsigned char *, unsigned int);
 int pkt_com_query(unsigned char *, unsigned int);
 enum MySQL_response_type mysql_response(unsigned char *, unsigned int);
 
-
-void pre_variable_mysql_threads(global_variable_entry_t *);
-
 void proxy_error_func(const char *, ...);
 
 #ifdef DEBUG
@@ -135,7 +111,6 @@ void init_debug_struct_from_cmdline();
 void proxy_debug_func(enum debug_module, int, int, const char *, int, const char *, const char *, ...);
 #endif
 
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
