@@ -9,8 +9,6 @@
 typedef struct { uint32_t hash; uint32_t key; } t_symstruct;
 
 
-
-
 class Scheduler_Row {
 	public:
 	unsigned int id;
@@ -19,25 +17,23 @@ class Scheduler_Row {
 	unsigned long long last;
 	unsigned long long next;
 	char *filename;
-//	char *arg1;
-//	char *arg2;
-//	char *arg3;
-//	char *arg4;
-//	char *arg5;
 	char **args;
 	char *comment;
 	Scheduler_Row(unsigned int _id, bool _is_active, unsigned int _in, char *_f, char *a1, char *a2, char *a3, char *a4, char *a5, char *_comment);
 	~Scheduler_Row();
 };
 
-
 class ProxySQL_External_Scheduler {
 	private:
 	unsigned long long next_run;
 	public:
 	unsigned int last_version;
 	unsigned int version;
+#ifdef PA_PTHREAD_MUTEX
+	pthread_rwlock_t rwlock;
+#else
 	rwlock_t rwlock;
+#endif
 	std::vector<Scheduler_Row *> Scheduler_Rows;
 	ProxySQL_External_Scheduler();
 	~ProxySQL_External_Scheduler();
@@ -45,7 +41,6 @@ class ProxySQL_External_Scheduler {
 	void update_table(SQLite3_result *result);
 };
 
-
 class ProxySQL_Admin {
 	private:
 	volatile int main_shutdown;
@@ -54,7 +49,6 @@ class ProxySQL_Admin {
 	std::vector<table_def_t *> *tables_defs_stats;
 	std::vector<table_def_t *> *tables_defs_config;
 
-
 	pthread_t admin_thr;
 
 	int main_poll_nfds;
@@ -62,7 +56,11 @@ class ProxySQL_Admin {
 	int *main_callback_func;
 
 
+#ifdef PA_PTHREAD_MUTEX
+	pthread_rwlock_t rwlock;
+#else
 	rwlock_t rwlock;
+#endif
 
 #ifdef PA_PTHREAD_MUTEX
 	pthread_mutex_t mysql_servers_lock;
@@ -86,6 +84,18 @@ class ProxySQL_Admin {
 		bool admin_read_only;
 		bool hash_passwords;
 		char * admin_version;
+		char * cluster_username;
+		char * cluster_password;
+		int cluster_check_interval_ms;
+		int cluster_check_status_frequency;
+		int cluster_mysql_query_rules_diffs_before_sync;
+		int cluster_mysql_servers_diffs_before_sync;
+		int cluster_mysql_users_diffs_before_sync;
+		int cluster_proxysql_servers_diffs_before_sync;
+		bool cluster_mysql_query_rules_save_to_disk;
+		bool cluster_mysql_servers_save_to_disk;
+		bool cluster_mysql_users_save_to_disk;
+		bool cluster_proxysql_servers_save_to_disk;
 #ifdef DEBUG
 		bool debug;
 #endif /* DEBUG */
@@ -98,7 +108,6 @@ class ProxySQL_Admin {
 	void insert_into_tables_defs(std::vector<table_def_t *> *, const char *table_name, const char *table_def);
 	void drop_tables_defs(std::vector<table_def_t *> *tables_defs);
 	void check_and_build_standard_tables(SQLite3DB *db, std::vector<table_def_t *> *tables_defs);
-	//void fill_table__server_status(SQLite3DB *db);
 
 #ifdef DEBUG
 	void flush_debug_levels_runtime_to_database(SQLite3DB *db, bool replace);
@@ -109,20 +118,16 @@ class ProxySQL_Admin {
 	void __insert_or_replace_maintable_select_disktable();
 	void __delete_disktable();
 	void __insert_or_replace_disktable_select_maintable();
-//	void __attach_configdb_to_admindb();
 	void __attach_db(SQLite3DB *db1, SQLite3DB *db2, char *alias);
 
-
-	void __add_active_users(enum cred_username_type usertype);
+	void __add_active_users(enum cred_username_type usertype, char *user=NULL, uint64_t *hash1 = NULL);
 	void __delete_inactive_users(enum cred_username_type usertype);
-//	void add_default_user(char *, char *);
 	void add_admin_users();
 	void __refresh_users();
 
 	void flush_mysql_variables___runtime_to_database(SQLite3DB *db, bool replace, bool del, bool onlyifempty, bool runtime=false);
 	void flush_mysql_variables___database_to_runtime(SQLite3DB *db, bool replace);
 
-
 	char **get_variables_list();
 	char *get_variable(char *name);
 	bool set_variable(char *name, char *value);
@@ -130,6 +135,7 @@ class ProxySQL_Admin {
 	void flush_admin_variables___runtime_to_database(SQLite3DB *db, bool replace, bool del, bool onlyifempty, bool runtime=false);
 	void disk_upgrade_mysql_query_rules();
 	void disk_upgrade_mysql_servers();
+	void disk_upgrade_mysql_users();
 	void disk_upgrade_scheduler();
 
 #ifdef DEBUG
@@ -139,13 +145,33 @@ class ProxySQL_Admin {
 	void add_credentials(char *credentials, int hostgroup_id);
 	void delete_credentials(char *credentials);
 #endif /* DEBUG */
+
+#ifdef PROXYSQLCLICKHOUSE
+	// ClickHouse
+	void __refresh_clickhouse_users();
+	void __add_active_clickhouse_users(char *user=NULL);
+	void __delete_inactive_clickhouse_users();
+	void flush_clickhouse_variables___runtime_to_database(SQLite3DB *db, bool replace, bool del, bool onlyifempty, bool runtime=false);
+	void flush_clickhouse_variables___database_to_runtime(SQLite3DB *db, bool replace);
+#endif /* PROXYSQLCLICKHOUSE */
+
+	void flush_sqliteserver_variables___runtime_to_database(SQLite3DB *db, bool replace, bool del, bool onlyifempty, bool runtime=false);
+	void flush_sqliteserver_variables___database_to_runtime(SQLite3DB *db, bool replace);
+	
+
 	public:
 	struct {
-		//re2::RE2::Options *opt;
-		//RE2 *re1;
 		void *opt;
 		void **re;
 	} match_regexes;
+	struct {
+		bool checksum_mysql_query_rules;
+		bool checksum_mysql_servers;
+		bool checksum_mysql_users;
+	} checksum_variables;
+	void public_add_active_users(enum cred_username_type usertype, char *user=NULL) {
+		__add_active_users(usertype, user);
+	}
 	ProxySQL_Admin();
 	~ProxySQL_Admin();
 	SQLite3DB *admindb;	// in memory
@@ -161,14 +187,13 @@ class ProxySQL_Admin {
 	void init_users();
 	void init_mysql_servers();
 	void init_mysql_query_rules();
+	void init_proxysql_servers();
 	void save_mysql_users_runtime_to_database(bool _runtime);
 	void save_mysql_servers_runtime_to_database(bool);
 	void admin_shutdown();
 	bool is_command(std::string);
-//	void SQLite3_to_MySQL(SQLite3_result *result, char *error, int affected_rows, MySQL_Protocol *myprot);
 	void send_MySQL_OK(MySQL_Protocol *myprot, char *msg, int rows=0);
 	void send_MySQL_ERR(MySQL_Protocol *myprot, char *msg);
-//	virtual void admin_session_handler(MySQL_Session *sess);
 #ifdef DEBUG
 	int load_debug_to_runtime() { return flush_debug_levels_database_to_runtime(admindb); }
 	void save_debug_from_runtime() { return flush_debug_levels_runtime_to_database(admindb, true); }
@@ -177,8 +202,8 @@ class ProxySQL_Admin {
 	void flush_mysql_users__from_disk_to_memory();
 	void flush_mysql_servers__from_memory_to_disk();
 	void flush_mysql_servers__from_disk_to_memory();
-	void flush_mysql_query_rules__from_memory_to_disk();	
-	void flush_mysql_query_rules__from_disk_to_memory();	
+	void flush_mysql_query_rules__from_memory_to_disk();
+	void flush_mysql_query_rules__from_disk_to_memory();
 	void load_mysql_servers_to_runtime();
 	void save_mysql_servers_from_runtime();
 	char * load_mysql_query_rules_to_runtime();
@@ -195,20 +220,25 @@ class ProxySQL_Admin {
 	void load_mysql_variables_to_runtime() { flush_mysql_variables___database_to_runtime(admindb, true); }
 	void save_mysql_variables_from_runtime() { flush_mysql_variables___runtime_to_database(admindb, true, true, false); }
 
-
 	void stats___mysql_query_rules();
 	void stats___mysql_query_digests(bool reset);
 	//void stats___mysql_query_digests_reset();
 	void stats___mysql_commands_counters();
 	void stats___mysql_processlist();
-	void stats___mysql_connection_pool();
+	void stats___mysql_connection_pool(bool _reset);
+	void stats___memory_metrics();
 	void stats___mysql_global();
+	void stats___mysql_users();
+
+	void stats___proxysql_servers_checksums();
+	void stats___proxysql_servers_metrics();
 
 	int Read_Global_Variables_from_configfile(const char *prefix);
 	int Read_MySQL_Users_from_configfile();
 	int Read_MySQL_Query_Rules_from_configfile();
 	int Read_MySQL_Servers_from_configfile();
 	int Read_Scheduler_from_configfile();
+	int Read_ProxySQL_Servers_from_configfile();
 
 	void flush_error_log();
 	void GenericRefreshStatistics(const char *query_no_space, unsigned int query_no_space_length, bool admin);
@@ -221,5 +251,30 @@ class ProxySQL_Admin {
 	// wrapper to call a private function
 	unsigned long long scheduler_run_once() { return scheduler->run_once(); }
 
+	void flush_configdb(); // 923
+
+	// Cluster
+	void load_proxysql_servers_to_runtime(bool _lock=true);
+	void flush_proxysql_servers__from_memory_to_disk();
+	void flush_proxysql_servers__from_disk_to_memory();
+	void save_proxysql_servers_runtime_to_database(bool);
+	void dump_checksums_values_table();
+
+	// SQLite Server
+	void init_sqliteserver_variables();
+	void load_sqliteserver_variables_to_runtime() { flush_sqliteserver_variables___database_to_runtime(admindb, true); }
+	void save_sqliteserver_variables_from_runtime() { flush_sqliteserver_variables___runtime_to_database(admindb, true, true, false); }
+
+#ifdef PROXYSQLCLICKHOUSE
+	// ClickHouse
+	void init_clickhouse_variables();
+	void load_clickhouse_variables_to_runtime() { flush_clickhouse_variables___database_to_runtime(admindb, true); }
+	void save_clickhouse_variables_from_runtime() { flush_clickhouse_variables___runtime_to_database(admindb, true, true, false); }
+	void init_clickhouse_users();
+	void flush_clickhouse_users__from_memory_to_disk();
+	void flush_clickhouse_users__from_disk_to_memory();
+	void save_clickhouse_users_runtime_to_database(bool _runtime);
+#endif /* PROXYSQLCLICKHOUSE */
+
 };
 #endif /* __CLASS_PROXYSQL_ADMIN_H */
