@@ -1,25 +1,47 @@
 #ifndef __CLASS_PROXYSQL_GLOVARS_H
 #define __CLASS_PROXYSQL_GLOVARS_H
-//#include "proxysql.h"
-//#include "cpp.h"
 
-//#include "ezOptionParser.hpp"
 #include "configfile.hpp"
+#include "proxy_defines.h"
 
 namespace ez {
 class ezOptionParser;
 };
 
+class ProxySQL_Checksum_Value {
+	public:
+	char *checksum;
+	unsigned long long version;
+	unsigned long long epoch;
+	ProxySQL_Checksum_Value() {
+		checksum = (char *)malloc(20);
+		memset(checksum,0,20);
+		version = 0;
+		epoch = 0;
+	}
+	void set_checksum(char *c) {
+		memset(checksum,0,20);
+		strncpy(checksum,c,18);
+		for (int i=2; i<18; i++) {
+			if (checksum[i]==' ' || checksum[i]==0) {
+				checksum[i]='0';
+			}
+		}
+
+	}
+	~ProxySQL_Checksum_Value() {
+		free(checksum);
+		checksum = NULL;
+	}
+};
+
 class ProxySQL_GlobalVariables {
 	public:
 	ez::ezOptionParser *opt;
-	//ezOptionParser *opt;
 	ProxySQL_ConfigFile *confFile;
 	bool configfile_open;
 	char *__cmd_proxysql_config_file;
 	char *__cmd_proxysql_datadir;
-	//char *__cmd_proxysql_admin_pathdb;
-//	bool __cmd_proxysql_print_version=false;
 	int __cmd_proxysql_nostart;
 	int __cmd_proxysql_foreground;
 	int __cmd_proxysql_gdbg;
@@ -39,7 +61,9 @@ class ProxySQL_GlobalVariables {
 #ifdef SO_REUSEPORT
 		bool reuseport;
 #endif /* SO_REUSEPORT */
-//		bool use_proxysql_mem;
+#ifdef IDLE_THREADS
+		bool idle_threads;
+#endif /* IDLE_THREADS */
 		pthread_mutex_t start_mutex;
 		bool foreground;
 #ifdef DEBUG
@@ -52,13 +76,34 @@ class ProxySQL_GlobalVariables {
 		bool restart_on_error;
 		int restart_delay;
 		SSL_CTX *ssl_ctx;	
+		bool sqlite3_server;
+#ifdef PROXYSQLCLICKHOUSE
+		bool clickhouse_server;
+#endif /* PROXYSQLCLICKHOUSE */
 	} global;
 	struct mysql {
 		char *server_version;
 		int poll_timeout;
 	};
-	//MySQL_HostGroups *MyHostGroups;
-	//std::vector<MySQL_Hostgroup *> MyHostGroups;
+	struct {
+		unsigned long stack_memory_mysql_threads;
+		unsigned long stack_memory_admin_threads;
+		unsigned long stack_memory_cluster_threads;
+	} statuses;
+	pthread_mutex_t checksum_mutex;
+	time_t epoch_version;
+	struct {
+		ProxySQL_Checksum_Value admin_variables;
+		ProxySQL_Checksum_Value mysql_query_rules;
+		ProxySQL_Checksum_Value mysql_servers;
+		ProxySQL_Checksum_Value mysql_users;
+		ProxySQL_Checksum_Value mysql_variables;
+		ProxySQL_Checksum_Value proxysql_servers;
+		uint64_t global_checksum;
+		unsigned long long updates_cnt;
+		unsigned long long dumped_at;
+	} checksums_values;
+	uint64_t generate_global_checksum();
 	ProxySQL_GlobalVariables();
 	~ProxySQL_GlobalVariables();
 	void process_opts_pre();
