@@ -3,69 +3,10 @@
 #include "proxysql.h"
 #include "cpp.h"
 
+#define PROXYSQL_QPRO_PTHREAD_MUTEX
 
-//typedef btree::btree_map<uint64_t, void *> BtMap_query_digest;
 typedef std::unordered_map<std::uint64_t, void *> umap_query_digest;
 
-/*
-enum MYSQL_COM_QUERY_command {
-	MYSQL_COM_QUERY_ALTER_TABLE,
-	MYSQL_COM_QUERY_ALTER_VIEW,
-	MYSQL_COM_QUERY_ANALYZE_TABLE,
-	MYSQL_COM_QUERY_BEGIN,
-	MYSQL_COM_QUERY_CALL,
-	MYSQL_COM_QUERY_CHANGE_MASTER,
-	MYSQL_COM_QUERY_COMMIT,
-	MYSQL_COM_QUERY_CREATE_DATABASE,
-	MYSQL_COM_QUERY_CREATE_INDEX,
-	MYSQL_COM_QUERY_CREATE_TABLE,
-	MYSQL_COM_QUERY_CREATE_TEMPORARY,
-	MYSQL_COM_QUERY_CREATE_TRIGGER,
-	MYSQL_COM_QUERY_CREATE_USER,
-	MYSQL_COM_QUERY_CREATE_VIEW,
-	MYSQL_COM_QUERY_DEALLOCATE,
-	MYSQL_COM_QUERY_DELETE,
-	MYSQL_COM_QUERY_DESCRIBE,
-	MYSQL_COM_QUERY_DROP_DATABASE,
-	MYSQL_COM_QUERY_DROP_INDEX,
-	MYSQL_COM_QUERY_DROP_TABLE,
-	MYSQL_COM_QUERY_DROP_TRIGGER,
-	MYSQL_COM_QUERY_DROP_USER,
-	MYSQL_COM_QUERY_DROP_VIEW,
-	MYSQL_COM_QUERY_GRANT,
-	MYSQL_COM_QUERY_EXECUTE,
-	MYSQL_COM_QUERY_EXPLAIN,
-	MYSQL_COM_QUERY_FLUSH,
-	MYSQL_COM_QUERY_INSERT,
-	MYSQL_COM_QUERY_KILL,
-	MYSQL_COM_QUERY_LOAD,
-	MYSQL_COM_QUERY_LOCK_TABLE,
-	MYSQL_COM_QUERY_OPTIMIZE,
-	MYSQL_COM_QUERY_PREPARE,
-	MYSQL_COM_QUERY_PURGE,
-	MYSQL_COM_QUERY_RENAME_TABLE,
-	MYSQL_COM_QUERY_RESET_MASTER,
-	MYSQL_COM_QUERY_RESET_SLAVE,
-	MYSQL_COM_QUERY_REPLACE,
-	MYSQL_COM_QUERY_REVOKE,
-	MYSQL_COM_QUERY_ROLLBACK,
-	MYSQL_COM_QUERY_SAVEPOINT,
-	MYSQL_COM_QUERY_SELECT,
-	MYSQL_COM_QUERY_SELECT_FOR_UPDATE,
-	MYSQL_COM_QUERY_SET,
-	MYSQL_COM_QUERY_SHOW_TABLE_STATUS,
-	MYSQL_COM_QUERY_START_TRANSACTION,
-	MYSQL_COM_QUERY_TRUNCATE_TABLE,
-	MYSQL_COM_QUERY_UNLOCK_TABLES,
-	MYSQL_COM_QUERY_UPDATE,
-	MYSQL_COM_QUERY_USE,
-	MYSQL_COM_QUERY_SHOW,
-	MYSQL_COM_QUERY_UNKNOWN,
-	MYSQL_COM_QUERY___NONE // Special marker.
-};
-
-*/
-
 struct _Query_Processor_rule_t {
 	int rule_id;
 	bool active;
@@ -79,6 +20,7 @@ struct _Query_Processor_rule_t {
 	char *match_digest;
 	char *match_pattern;
 	bool negate_match_pattern;
+	int re_modifiers; // note: this is passed as char*, but converted to bitsfield
 	int flagOUT;
 	char *replace_pattern;
 	int destination_hostgroup;
@@ -87,9 +29,13 @@ struct _Query_Processor_rule_t {
 	int timeout;
 	int retries;
 	int delay;
+	int next_query_flagIN;
 	int mirror_hostgroup;
 	int mirror_flagOUT;
 	char *error_msg;
+	char *OK_msg;
+	int sticky_conn;
+	int multiplex;
 	int log;
 	bool apply;
   char *comment; // #643
@@ -99,20 +45,7 @@ struct _Query_Processor_rule_t {
 	struct _Query_Processor_rule_t *parent; // pointer to parent, to speed up parent update
 };
 
-
-//struct _Query_Processor_output_t {
-//	void *ptr;
-//	unsigned int size;
-//	int destination_hostgroup;
-//	int cache_ttl;
-//	int reconnect;
-//	int timeout;
-//	int delay;
-//	std::string *new_query;
-//};
-
 typedef struct _Query_Processor_rule_t QP_rule_t;
-//typedef struct _Query_Processor_output_t QP_out_t;
 
 class Query_Processor_Output {
 	public:
@@ -121,12 +54,16 @@ class Query_Processor_Output {
 	int destination_hostgroup;
 	int mirror_hostgroup;
 	int mirror_flagOUT;
+	int next_query_flagIN;
 	int cache_ttl;
 	int reconnect;
 	int timeout;
 	int retries;
 	int delay;
-  char *error_msg;
+	char *error_msg;
+	char *OK_msg;
+	int sticky_conn;
+	int multiplex;
 	int log;
   char *comment; // #643
 	std::string *new_query;
@@ -148,14 +85,18 @@ class Query_Processor_Output {
 		destination_hostgroup=-1;
 		mirror_hostgroup=-1;
 		mirror_flagOUT=-1;
+		next_query_flagIN=-1;
 		cache_ttl=-1;
 		reconnect=-1;
 		timeout=-1;
 		retries=-1;
 		delay=-1;
+		sticky_conn=-1;
+		multiplex=-1;
 		log=-1;
 		new_query=NULL;
 		error_msg=NULL;
+		OK_msg=NULL;
 		comment=NULL; // #643
 	}
 	void destroy() {
@@ -163,6 +104,10 @@ class Query_Processor_Output {
 			free(error_msg);
 			error_msg=NULL;
 		}
+		if (OK_msg) {
+			free(OK_msg);
+			OK_msg=NULL;
+		}
 		if (comment) { // #643
 			free(comment);
 		}
@@ -222,35 +167,35 @@ class Command_Counter {
 	}
 };
 
-
-
 class Query_Processor {
-
-
 	private:
 	umap_query_digest digest_umap;
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_t digest_rwlock;
+#else
 	rwlock_t digest_rwlock;
 	int64_t padding;	// to get rwlock cache aligned
+#endif
 	enum MYSQL_COM_QUERY_command __query_parser_command_type(SQP_par_t *qp);
 	protected:
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_t rwlock;
+#else
 	rwlock_t rwlock;
+#endif
 	std::vector<QP_rule_t *> rules;
 	Command_Counter * commands_counters[MYSQL_COM_QUERY___NONE];
 	volatile unsigned int version;
 	public:
 	Query_Processor();
 	~Query_Processor();
-	//const char *version();
 	void print_version();
 	void reset_all(bool lock=true);
 	void wrlock();		// explicit write lock, to be used in multi-isert 
 	void wrunlock();	// explicit write unlock
 	bool insert(QP_rule_t *qr, bool lock=true);		// insert a new rule. Uses a generic void pointer to a structure that may vary depending from the Query Processor
-//	virtual bool insert_locked(QP_rule_t *qr) {return false;};		// call this instead of insert() in case lock was already acquired via wrlock()
-	QP_rule_t * new_query_rule(int rule_id, bool active, char *username, char *schemaname, int flagIN, char *client_addr, char *proxy_addr, int proxy_port, char *digest, char *match_digest, char *match_pattern, bool negate_match_pattern, int flagOUT, char *replace_pattern, int destination_hostgroup, int cache_ttl, int reconnect, int timeout, int retries, int delay, int mirror_hostgroup, int mirror_flagOUT, char *error_msg, int log, bool apply, char *comment);	// to use a generic query rule struct, this is generated by this function and returned as generic void pointer
+	QP_rule_t * new_query_rule(int rule_id, bool active, char *username, char *schemaname, int flagIN, char *client_addr, char *proxy_addr, int proxy_port, char *digest, char *match_digest, char *match_pattern, bool negate_match_pattern, char *re_modifiers, int flagOUT, char *replace_pattern, int destination_hostgroup, int cache_ttl, int reconnect, int timeout, int retries, int delay, int next_query_flagIN, int mirror_hostgroup, int mirror_flagOUT, char *error_msg, char *OK_msg, int sticky_conn, int multiplex, int log, bool apply, char *comment);	// to use a generic query rule struct, this is generated by this function and returned as generic void pointer
 	void delete_query_rule(QP_rule_t *qr);	// destructor
-	//virtual bool remove(int rule_id, bool lock=true) {return false;}; // FIXME: not implemented yet, should be implemented at all ?
-//	virtual bool remove_locked(int rule_id) {return false;};		// call this instead of remove() in case lock was already acquired via wrlock()
 	Query_Processor_Output * process_mysql_query(MySQL_Session *sess, void *ptr, unsigned int size, Query_Info *qi);
 	void delete_QP_out(Query_Processor_Output *o);
 
@@ -278,8 +223,10 @@ class Query_Processor {
 	SQLite3_result * get_stats_commands_counters();
 	SQLite3_result * get_query_digests();
 	SQLite3_result * get_query_digests_reset();
-};
 
+	unsigned long get_query_digests_total_size();
+
+};
 
 typedef Query_Processor * create_Query_Processor_t();
 
