@@ -4,10 +4,19 @@
 extern MySQL_Authentication *GloMyAuth;
 extern MySQL_Threads_Handler *GloMTH;
 
+#ifdef PROXYSQLCLICKHOUSE
+extern ClickHouse_Authentication *GloClickHouseAuth;
+#endif /* PROXYSQLCLICKHOUSE */
+
 #ifdef max_allowed_packet
 #undef max_allowed_packet
 #endif
 
+#if defined(__FreeBSD__) || defined(__APPLE__)
+typedef uint8_t uint8;
+typedef uint8_t uchar;
+#endif
+
 //#define RESULTSET_BUFLEN 16300
 
 
@@ -39,7 +48,6 @@ static void __dump_pkt(const char *func, unsigned char *_ptr, unsigned int len)
 }
 #endif
 
-
 char *sha1_pass_hex(char *sha1_pass) {
 	if (sha1_pass==NULL) return NULL;
 	char *buff=(char *)malloc(SHA_DIGEST_LENGTH*2+2);
@@ -81,7 +89,6 @@ static inline int write_encoded_length(unsigned char *p, uint64_t val, uint8_t l
 	return len;
 }
 
-
 static inline int write_encoded_length_and_string(unsigned char *p, uint64_t val, uint8_t len, char prefix, char *string) {
 	int l=write_encoded_length(p,val,len,prefix);
 	if (val) {
@@ -90,8 +97,6 @@ static inline int write_encoded_length_and_string(unsigned char *p, uint64_t val
 	return l+val;
 }
 
-
-
 void proxy_compute_sha1_hash_multi(uint8 *digest, const char *buf1, int len1, const char *buf2, int len2) {
   PROXY_TRACE();
   
@@ -102,7 +107,6 @@ void proxy_compute_sha1_hash_multi(uint8 *digest, const char *buf1, int len1, co
   SHA1_Final(digest, &sha1_context);
 }
 
-
 void proxy_compute_sha1_hash(uint8 *digest, const char *buf, int len) {
   PROXY_TRACE();
   
@@ -117,15 +121,12 @@ void proxy_compute_two_stage_sha1_hash(const char *password, size_t pass_len, ui
   proxy_compute_sha1_hash(hash_stage2, (const char *) hash_stage1, SHA_DIGEST_LENGTH);
 }
 
-
 void proxy_my_crypt(char *to, const uchar *s1, const uchar *s2, uint len) {
   const uint8 *s1_end= s1 + len;
   while (s1 < s1_end)
     *to++= *s1++ ^ *s2++;
 }
 
-
-
 unsigned char decode_char(char x) {
 	if (x >= '0' && x <= '9')
 		return (x - 0x30);
@@ -139,8 +140,6 @@ unsigned char decode_char(char x) {
 	}
 }
 
-
-
 void unhex_pass(uint8_t *out, const char *in) {
 	int i=0;
 	for (i=0;i<SHA_DIGEST_LENGTH;i++) {
@@ -155,28 +154,16 @@ void unhex_pass(uint8_t *out, const char *in) {
 	}
 }
 
-
-
-
 void proxy_scramble(char *to, const char *message, const char *password)
 {
 	uint8 hash_stage1[SHA_DIGEST_LENGTH];
 	uint8 hash_stage2[SHA_DIGEST_LENGTH];
-
-//	if (password[0]=='*') {
-//		// the password is a SHA1(SHA1(real_password))
-//		unhex_pass(hash_stage2,password+1);
-//	} else {
-		/* Two stage SHA1 hash of the password. */
 	proxy_compute_two_stage_sha1_hash(password, strlen(password), hash_stage1, hash_stage2);
-//	}
-	/* create crypt string as sha1(message, hash_stage2) */;
 	proxy_compute_sha1_hash_multi((uint8 *) to, message, SCRAMBLE_LENGTH, (const char *) hash_stage2, SHA_DIGEST_LENGTH);
 	proxy_my_crypt(to, (const uchar *) to, hash_stage1, SCRAMBLE_LENGTH);
 	return;
 }
 
-
 bool proxy_scramble_sha1(char *pass_reply,  const char *message, const char *sha1_sha1_pass, char *sha1_pass) {
 	bool ret=false;
 	uint8 hash_stage1[SHA_DIGEST_LENGTH];
@@ -194,11 +181,6 @@ bool proxy_scramble_sha1(char *pass_reply,  const char *message, const char *sha
 	return ret;
 }
 
-
-
-
-
-
 typedef union _4bytes_t {
 	unsigned char data[4];
 	uint32_t i;
@@ -206,16 +188,11 @@ typedef union _4bytes_t {
 
 unsigned int CPY3(unsigned char *ptr) {
 	_4bytes_t buf;
-//	memcpy(buf.data, pkt, 3);	
 	buf.i=*(uint32_t *)ptr;
 	buf.data[3]=0;
-//	unsigned char _cpy3buf[4];
-//	_cpy3buf[3]=0;
-//	unsigned int ret=*(unsigned int *)_cpy3buf;
 	return buf.i;
 }
 
-
 // see http://dev.mysql.com/doc/internals/en/integer.html#packet-Protocol::LengthEncodedInteger
 /* arguments to pass:
  * pointer to the field
@@ -323,14 +300,6 @@ int pkt_end(unsigned char *pkt, unsigned int length, MySQL_Protocol *mp)
 #endif /* DEBUG */
 		pkt    += 2;
 		mp->prot_status  = CPY2(pkt);
-
-/*
-      if((tag->state == STATE_TXT_ROW || tag->state == STATE_BIN_ROW) &&
-         status & SERVER_MORE_RESULTS_EXISTS &&
-         tag->event != EVENT_END_MULTI_RESULT)
-            return PKT_WRONG_TYPE;
-   }
-*/
 	}
 	proxy_debug(PROXY_DEBUG_MYSQL_PROTOCOL,1,"End Packet <status:%u warns:%u>\n", mp->prot_status, warns);
 
@@ -365,14 +334,10 @@ void MySQL_Protocol::init(MySQL_Data_Stream **__myds, MySQL_Connection_userinfo
 	userinfo=__userinfo;
 	sess=__sess;
 	current_PreStmt=NULL;
-//	prot_status=0;
 }
 
-//int parse_mysql_pkt(unsigned char *pkt, enum session_states *states, int from_client) {
 int MySQL_Protocol::parse_mysql_pkt(PtrSize_t *PS_entry, MySQL_Data_Stream *__myds) {
 	unsigned char *pkt=(unsigned char *)PS_entry->ptr;	
-//	unsigned int size=PS_entry->size;
-	//myds=__myds;
 	enum mysql_data_stream_status *DSS=&(*myds)->DSS;
 
 	mysql_hdr hdr;
@@ -383,7 +348,6 @@ int MySQL_Protocol::parse_mysql_pkt(PtrSize_t *PS_entry, MySQL_Data_Stream *__my
 
 	payload=pkt+sizeof(mysql_hdr);
 	memcpy(&hdr,pkt,sizeof(mysql_hdr));
-	//Copy4B(&hdr,pkt);
 	proxy_debug(PROXY_DEBUG_MYSQL_PROTOCOL,1,"MySQL Packet length=%d, senquence_id=%d, addr=%p\n", hdr.pkt_length, hdr.pkt_id, payload);
 
 	switch (*DSS) {
@@ -429,9 +393,6 @@ int MySQL_Protocol::parse_mysql_pkt(PtrSize_t *PS_entry, MySQL_Data_Stream *__my
 			}
 			//break;
 
-
-
-			
 		default:
 		// TO BE REMOVED: begin
 			if (from==MYDS_FRONTEND) { // at this stage we expect a packet from the server, not from client
@@ -462,33 +423,23 @@ int MySQL_Protocol::parse_mysql_pkt(PtrSize_t *PS_entry, MySQL_Data_Stream *__my
 
 
 static unsigned char protocol_version=10;
-//static uint16_t server_capabilities=CLIENT_FOUND_ROWS | CLIENT_PROTOCOL_41 | CLIENT_IGNORE_SIGPIPE | CLIENT_TRANSACTIONS | CLIENT_SECURE_CONNECTION | CLIENT_CONNECT_WITH_DB | CLIENT_SSL;
-//static uint8_t server_language=33;
 static uint16_t server_status=SERVER_STATUS_AUTOCOMMIT;
-//static char *mysql_server_version = (char *)"5.1.30";
 
-//bool MySQL_Protocol::generate_statistics_response(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len) {
 bool MySQL_Protocol::generate_statistics_response(bool send, void **ptr, unsigned int *len) {
 // FIXME : this function generates a not useful string. It is a placeholder for now
 
 	char buf1[1000];
 	unsigned long long t1=monotonic_time();
-	//const char *stats=(char *)"Uptime: 1000  Threads: 1  Questions: 34221015  Slow queries: 0  Opens: 757  Flush tables: 1  Open tables: 185  Queries per second avg: 22.289";
 	sprintf(buf1,"Uptime: %llu Threads: %d  Questions: %llu  Slow queries: %llu", (t1-GloVars.global.start_time)/1000/1000, MyHGM->status.client_connections , GloMTH->get_total_queries() , GloMTH->get_slow_queries() );
 	unsigned char statslen=strlen(buf1);
 	mysql_hdr myhdr;
 	myhdr.pkt_id=1;
-	//myhdr.pkt_length=statslen+1;
 	myhdr.pkt_length=statslen;
-	
 
-	
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
   unsigned char *_ptr=(unsigned char *)l_alloc(size);
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
-  //Copy4B(_ptr, &myhdr);
   int l=sizeof(mysql_hdr);
-	//_ptr[l++]=statslen;
 	memcpy(_ptr+l,buf1,statslen);
 
 	if (send==true) { (*myds)->PSarrayOUT->add((void *)_ptr,size); }
@@ -500,7 +451,6 @@ bool MySQL_Protocol::generate_statistics_response(bool send, void **ptr, unsigne
 	return true;
 }
 
-//bool MySQL_Protocol::generate_pkt_EOF(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t warnings, uint16_t status) {
 bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t warnings, uint16_t status) {
 	if ((*myds)->sess->mirror==true) {
 		return true;
@@ -511,7 +461,6 @@ bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len,
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
   unsigned char *_ptr=(unsigned char *)l_alloc(size);
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
-  //Copy4B(_ptr, &myhdr);
   int l=sizeof(mysql_hdr);
 	_ptr[l]=0xfe; l++;
 	memcpy(_ptr+l, &warnings, sizeof(uint16_t)); l+=sizeof(uint16_t);
@@ -539,7 +488,6 @@ bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len,
 	return true;
 }
 
-//bool MySQL_Protocol::generate_pkt_ERR(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t error_code, char *sql_state, char *sql_message) {
 bool MySQL_Protocol::generate_pkt_ERR(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t error_code, char *sql_state, char *sql_message) {
 	if ((*myds)->sess->mirror==true) {
 		return true;
@@ -551,7 +499,6 @@ bool MySQL_Protocol::generate_pkt_ERR(bool send, void **ptr, unsigned int *len,
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
   unsigned char *_ptr=(unsigned char *)l_alloc(size);
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
-  //Copy4B(_ptr, &myhdr);
   int l=sizeof(mysql_hdr);
 	_ptr[l]=0xff; l++;
 	memcpy(_ptr+l, &error_code, sizeof(uint16_t)); l+=sizeof(uint16_t);
@@ -585,8 +532,6 @@ bool MySQL_Protocol::generate_pkt_ERR(bool send, void **ptr, unsigned int *len,
 	return true;
 }
 
-
-//bool MySQL_Protocol::generate_pkt_OK(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len, uint8_t sequence_id, unsigned int affected_rows, unsigned int last_insert_id, uint16_t status, uint16_t warnings, char *msg) {
 bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, unsigned int affected_rows, uint64_t last_insert_id, uint16_t status, uint16_t warnings, char *msg) {
 	if ((*myds)->sess->mirror==true) {
 		return true;
@@ -606,19 +551,8 @@ bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, u
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
   unsigned char *_ptr=(unsigned char *)l_alloc(size);
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
-  //Copy4B(_ptr, &myhdr);
   int l=sizeof(mysql_hdr);
 	_ptr[l]=0x00; l++;
-/*
-	if (affected_rows_len > 1) {
-		_ptr[l]=affected_rows_prefix; l++;
-	}
-	memcpy(_ptr+l, &affected_rows, affected_rows_len); l+=( affected_rows_len > 1 ? affected_rows_len - 1 : 1 ); 
-	if (last_insert_id_len > 1) {
-		_ptr[l]=last_insert_id_prefix; l++;
-	}
-	memcpy(_ptr+l, &last_insert_id, last_insert_id_len); l+=( last_insert_id_len > 1 ? last_insert_id_len -1 : 1 );
-*/
 	l+=write_encoded_length(_ptr+l, affected_rows, affected_rows_len, affected_rows_prefix);
 	l+=write_encoded_length(_ptr+l, last_insert_id, last_insert_id_len, last_insert_id_prefix);
 	memcpy(_ptr+l, &status, sizeof(uint16_t)); l+=sizeof(uint16_t);
@@ -649,7 +583,6 @@ bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, u
 	return true;
 }
 
-//bool MySQL_Protocol::generate_pkt_column_count(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint64_t count) {
 bool MySQL_Protocol::generate_pkt_column_count(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint64_t count) {
 	if ((*myds)->sess->mirror==true) {
 		return true;
@@ -664,17 +597,8 @@ bool MySQL_Protocol::generate_pkt_column_count(bool send, void **ptr, unsigned i
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
   unsigned char *_ptr=(unsigned char *)l_alloc(size);
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
-  //Copy4B(_ptr, &myhdr);
   int l=sizeof(mysql_hdr);
 
-
-/*
-	if (count_len > 1) {
-		_ptr[l]=count_prefix; l++;
-	}
-	memcpy(_ptr+l, &count, count_len); l+=( count_len > 1 ? count_len -1 : 1 );
-*/
-
 	l+=write_encoded_length(_ptr+l, count, count_len, count_prefix);
 
 	if (send==true) { (*myds)->PSarrayOUT->add((void *)_ptr,size); }
@@ -686,8 +610,6 @@ bool MySQL_Protocol::generate_pkt_column_count(bool send, void **ptr, unsigned i
 	return true;
 }
 
-
-//bool MySQL_Protocol::generate_pkt_field(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len, uint8_t sequence_id, char *schema, char *table, char *org_table, char *name, char *org_name, uint16_t charset, uint32_t column_length, uint8_t type, uint16_t flags, uint8_t decimals, bool field_list, uint64_t defvalue_length, char *defvalue) {
 bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, char *schema, char *table, char *org_table, char *name, char *org_name, uint16_t charset, uint32_t column_length, uint8_t type, uint16_t flags, uint8_t decimals, bool field_list, uint64_t defvalue_length, char *defvalue) {
 
 	if ((*myds)->sess->mirror==true) {
@@ -777,7 +699,7 @@ bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len
 
 // FIXME FIXME function not completed yet!
 // see https://dev.mysql.com/doc/internals/en/com-stmt-prepare-response.html
-bool MySQL_Protocol::generate_STMT_PREPARE_RESPONSE(uint8_t sequence_id, MySQL_STMT_Global_info *stmt_info) {
+bool MySQL_Protocol::generate_STMT_PREPARE_RESPONSE(uint8_t sequence_id, MySQL_STMT_Global_info *stmt_info, uint32_t _stmt_id) {
 	uint8_t sid=sequence_id;
 	uint16_t i;
 	char *okpack=(char *)malloc(16); // first packet
@@ -787,7 +709,12 @@ bool MySQL_Protocol::generate_STMT_PREPARE_RESPONSE(uint8_t sequence_id, MySQL_S
 	memcpy(okpack,&hdr,sizeof(mysql_hdr)); // copy header
 	okpack[4]=0;
 	okpack[13]=0;
-	memcpy(okpack+5,&stmt_info->statement_id,sizeof(uint32_t));
+	okpack[15]=0;
+	if (_stmt_id) {
+		memcpy(okpack+5,&_stmt_id,sizeof(uint32_t));
+	} else {
+		memcpy(okpack+5,&stmt_info->statement_id,sizeof(uint32_t));
+	}
 	memcpy(okpack+9,&stmt_info->num_columns,sizeof(uint16_t));
 	memcpy(okpack+11,&stmt_info->num_params,sizeof(uint16_t));
 	memcpy(okpack+14,&stmt_info->warning_count,sizeof(uint16_t));
@@ -806,7 +733,6 @@ bool MySQL_Protocol::generate_STMT_PREPARE_RESPONSE(uint8_t sequence_id, MySQL_S
 	if (stmt_info->num_columns) {
 		for (i=0; i<stmt_info->num_columns; i++) {
 			MYSQL_FIELD *fd=stmt_info->fields[i];
-			//bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, char *schema, char *table, char *org_table, char *name, char *org_name, uint16_t charset, uint32_t column_length, uint8_t type, uint16_t flags, uint8_t decimals, bool field_list, uint64_t defvalue_length, char *defvalue) {
 			generate_pkt_field(true,NULL,NULL,sid,
 				fd->db,
 				fd->table, fd->org_table,
@@ -998,22 +924,16 @@ bool MySQL_Protocol::generate_pkt_initial_handshake(bool send, void **ptr, unsig
     + (strlen("mysql_native_password")+1);
 
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
-  //mypkt->data=g_slice_alloc0(mypkt->length);
-  //mypkt->data=l_alloc0(thrLD->sfp, mypkt->length);
   unsigned char *_ptr=(unsigned char *)malloc(size);
 	memset(_ptr,0,size);
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
-  //Copy4B(_ptr, &myhdr);
   int l;
   l=sizeof(mysql_hdr);
-  //srand(pthread_self());
-  //uint32_t thread_id=rand()%100000;
   uint32_t thread_id=__sync_fetch_and_add(&glovars.thread_id,1);
 	if (thread_id==0) {
 		thread_id=__sync_fetch_and_add(&glovars.thread_id,1); // again!
 	}
 	*_thread_id=thread_id;
-  //uint32_t thread_id=pthread_self();
 
   rand_struct rand_st;
   //randominit(&rand_st,rand(),rand());
@@ -1042,7 +962,6 @@ bool MySQL_Protocol::generate_pkt_initial_handshake(bool send, void **ptr, unsig
   _ptr[l]=0x00; l+=1; //0x00
 	if (mysql_thread___have_compress) {
 		mysql_thread___server_capabilities |= CLIENT_COMPRESS; // FIXME: shouldn't be here
-		//(*myds)->myconn->options.compression_min_length=50;
 	}
 	(*myds)->myconn->options.server_capabilities=mysql_thread___server_capabilities;
   memcpy(_ptr+l,&mysql_thread___server_capabilities, sizeof(mysql_thread___server_capabilities)); l+=sizeof(mysql_thread___server_capabilities);
@@ -1081,8 +1000,6 @@ bool MySQL_Protocol::generate_pkt_initial_handshake(bool send, void **ptr, unsig
 	return true;
 }
 
-
-//bool MySQL_Protocol::process_pkt_OK(MySQL_Data_Stream *myds, unsigned char *pkt, unsigned int len) {
 bool MySQL_Protocol::process_pkt_OK(unsigned char *pkt, unsigned int len) {
 
   if (len < 11) return false;
@@ -1094,8 +1011,6 @@ bool MySQL_Protocol::process_pkt_OK(unsigned char *pkt, unsigned int len) {
 	if (*pkt) return false;
 	if (len!=hdr.pkt_length+sizeof(mysql_hdr)) return false;
 
-	//MYSQL &myc=(*myds)->myconn->myconn;
-
 	uint64_t affected_rows;
 	uint64_t  insert_id;
 #ifdef DEBUG
@@ -1106,7 +1021,6 @@ bool MySQL_Protocol::process_pkt_OK(unsigned char *pkt, unsigned int len) {
 	unsigned int p=0;
 	int rc;
 
-   //field_count = (u_int)*pkt++;
 	pkt++; p++;
 	rc=mysql_decode_length(pkt,&affected_rows);
 	pkt += rc; p+=rc;
@@ -1144,7 +1058,6 @@ bool MySQL_Protocol::process_pkt_EOF(unsigned char *pkt, unsigned int len) {
 	return ( ret==PKT_PARSED ? true : false );
 }
 
-//bool MySQL_Protocol::process_pkt_COM_QUERY(MySQL_Data_Stream *myds, unsigned char *pkt, unsigned int len) {
 bool MySQL_Protocol::process_pkt_COM_QUERY(unsigned char *pkt, unsigned int len) {
 	bool ret=false;
 
@@ -1181,14 +1094,18 @@ bool MySQL_Protocol::process_pkt_auth_swich_response(unsigned char *pkt, unsigne
 	char reply[SHA_DIGEST_LENGTH+1];
 	reply[SHA_DIGEST_LENGTH]='\0';
 	void *sha1_pass=NULL;
-	password=GloMyAuth->lookup((char *)userinfo->username, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, NULL, NULL, &transaction_persistent, NULL, NULL, &sha1_pass);
+	enum proxysql_session_type session_type = (*myds)->sess->session_type;
+	if (session_type == PROXYSQL_SESSION_CLICKHOUSE) {
+#ifdef PROXYSQLCLICKHOUSE
+		password=GloClickHouseAuth->lookup((char *)userinfo->username, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, NULL, NULL, &transaction_persistent, NULL, NULL, &sha1_pass);
+#endif /* PROXYSQLCLICKHOUSE */
+	} else {
+		password=GloMyAuth->lookup((char *)userinfo->username, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, NULL, NULL, &transaction_persistent, NULL, NULL, &sha1_pass);
+	}
 	// FIXME: add support for default schema and fast forward , issues #255 and #256
 	if (password==NULL) {
 		ret=false;
 	} else {
-//		if (pass_len==0 && strlen(password)==0) {
-//			ret=true;
-//		} else {
 			if (password[0]!='*') { // clear text password
 				proxy_scramble(reply, (*myds)->myconn->scramble_buff, password);
 				if (memcmp(reply, pass, SHA_DIGEST_LENGTH)==0) {
@@ -1205,10 +1122,6 @@ bool MySQL_Protocol::process_pkt_auth_swich_response(unsigned char *pkt, unsigne
 					userinfo->sha1_pass=sha1_pass_hex(reply);
 				}
 			}
-//		}
-//		if (_ret_use_ssl==true) {
-//			ret=false;
-//		}
 	}
 	if (sha1_pass) {
 		free(sha1_pass);
@@ -1245,7 +1158,14 @@ bool MySQL_Protocol::process_pkt_COM_CHANGE_USER(unsigned char *pkt, unsigned in
 	cur+=pass_len;
 	db=(char *)pkt+cur;
 	void *sha1_pass=NULL;
-	password=GloMyAuth->lookup((char *)user, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, NULL, NULL, &transaction_persistent, NULL, NULL, &sha1_pass);
+	enum proxysql_session_type session_type = (*myds)->sess->session_type;
+	if (session_type == PROXYSQL_SESSION_CLICKHOUSE) {
+#ifdef PROXYSQLCLICKHOUSE
+		password=GloClickHouseAuth->lookup((char *)user, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, NULL, NULL, &transaction_persistent, NULL, NULL, &sha1_pass);
+#endif /* PROXYSQLCLICKHOUSE */
+	} else {
+		password=GloMyAuth->lookup((char *)user, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, NULL, NULL, &transaction_persistent, NULL, NULL, &sha1_pass);
+	}
 	// FIXME: add support for default schema and fast forward, see issue #255 and #256
 	(*myds)->sess->default_hostgroup=default_hostgroup;
 	(*myds)->sess->transaction_persistent=transaction_persistent;
@@ -1259,7 +1179,9 @@ bool MySQL_Protocol::process_pkt_COM_CHANGE_USER(unsigned char *pkt, unsigned in
 				proxy_scramble(reply, (*myds)->myconn->scramble_buff, password);
 				if (memcmp(reply, pass, SHA_DIGEST_LENGTH)==0) {
 					ret=true;
-				} else {
+				}
+			} else {
+				if (session_type == PROXYSQL_SESSION_MYSQL || session_type == PROXYSQL_SESSION_SQLITE) {
 					ret=proxy_scramble_sha1((char *)pass,(*myds)->myconn->scramble_buff,password+1, reply);
 					if (ret) {
 						if (sha1_pass==NULL) {
@@ -1281,7 +1203,6 @@ bool MySQL_Protocol::process_pkt_COM_CHANGE_USER(unsigned char *pkt, unsigned in
 	if (userinfo->username) free(userinfo->username);
 	if (userinfo->password) free(userinfo->password);
 	if (ret==true) {
-		//(*myds)->myconn->options.max_allowed_pkt=max_pkt;
 		(*myds)->DSS=STATE_CLIENT_HANDSHAKE;
 
 		userinfo->username=strdup((const char *)user);
@@ -1292,7 +1213,6 @@ bool MySQL_Protocol::process_pkt_COM_CHANGE_USER(unsigned char *pkt, unsigned in
 		userinfo->username=strdup((const char *)user);
 		/*if (pass_len) */ userinfo->password=strdup((const char *)"");
 	}
-	//if (password) free(password);
 	if (password) {
 		free(password);
 		password=NULL;
@@ -1305,7 +1225,6 @@ bool MySQL_Protocol::process_pkt_COM_CHANGE_USER(unsigned char *pkt, unsigned in
 	return ret;
 }
 
-//bool MySQL_Protocol::process_pkt_handshake_response(MySQL_Data_Stream *myds, unsigned char *pkt, unsigned int len) {
 bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned int len) {
 	bool ret=false;
 	uint8_t charset;
@@ -1360,7 +1279,14 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 	bool transaction_persistent;
 	bool fast_forward;
 	int max_connections;
-	password=GloMyAuth->lookup((char *)user, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, &default_schema, &schema_locked, &transaction_persistent, &fast_forward, &max_connections, &sha1_pass);
+	enum proxysql_session_type session_type = (*myds)->sess->session_type;
+	if (session_type == PROXYSQL_SESSION_CLICKHOUSE) {
+#ifdef PROXYSQLCLICKHOUSE
+		password=GloClickHouseAuth->lookup((char *)user, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, &default_schema, &schema_locked, &transaction_persistent, &fast_forward, &max_connections, &sha1_pass);
+#endif /* PROXYSQLCLICKHOUSE */
+	} else {
+		password=GloMyAuth->lookup((char *)user, USERNAME_FRONTEND, &_ret_use_ssl, &default_hostgroup, &default_schema, &schema_locked, &transaction_persistent, &fast_forward, &max_connections, &sha1_pass);
+	}
 	//assert(default_hostgroup>=0);
 	(*myds)->sess->default_hostgroup=default_hostgroup;
 	(*myds)->sess->default_schema=default_schema; // just the pointer is passed
@@ -1370,7 +1296,11 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 	(*myds)->sess->user_max_connections=max_connections;
 	if (password==NULL) {
 		// this is a workaround for bug #603
-		if ((*myds)->sess->admin==true) {
+		if (
+			((*myds)->sess->session_type == PROXYSQL_SESSION_ADMIN)
+		|| 
+			((*myds)->sess->session_type == PROXYSQL_SESSION_STATS) 
+		) {
 			if (strcmp((const char *)user,mysql_thread___monitor_username)==0) {
 				proxy_scramble(reply, (*myds)->myconn->scramble_buff, mysql_thread___monitor_password);
 				if (memcmp(reply, pass, SHA_DIGEST_LENGTH)==0) {
@@ -1399,14 +1329,16 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 					ret=true;
 				}
 			} else {
-				ret=proxy_scramble_sha1((char *)pass,(*myds)->myconn->scramble_buff,password+1, reply);
-				if (ret) {
-					if (sha1_pass==NULL) {
-						// currently proxysql doesn't know any sha1_pass for that specific user, let's set it!
-						GloMyAuth->set_SHA1((char *)user, USERNAME_FRONTEND,reply);
+				if (session_type == PROXYSQL_SESSION_MYSQL || session_type == PROXYSQL_SESSION_SQLITE) {
+					ret=proxy_scramble_sha1((char *)pass,(*myds)->myconn->scramble_buff,password+1, reply);
+					if (ret) {
+						if (sha1_pass==NULL) {
+							// currently proxysql doesn't know any sha1_pass for that specific user, let's set it!
+							GloMyAuth->set_SHA1((char *)user, USERNAME_FRONTEND,reply);
+						}
+						if (userinfo->sha1_pass) free(userinfo->sha1_pass);
+						userinfo->sha1_pass=sha1_pass_hex(reply);
 					}
-					if (userinfo->sha1_pass) free(userinfo->sha1_pass);
-					userinfo->sha1_pass=sha1_pass_hex(reply);
 					}
 				}
 			}
@@ -1455,7 +1387,6 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 	}
 
 __exit_process_pkt_handshake_response:
-	//if (password) free(password);
 	if (password) {
 		free(password);
 		password=NULL;
@@ -1465,7 +1396,6 @@ __exit_process_pkt_handshake_response:
 		sha1_pass=NULL;
 	}
 
-	//l_free(len,pkt);
 	return ret;
 }
 
@@ -1514,9 +1444,6 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 	memcpy(&ret->flags,p,1); p+=1; // flags
 	p+=4; // iteration-count
 	ret->num_params=num_params;
-//	ret->binds=NULL;
-//	ret->is_nulls=NULL;
-//	ret->lengths=NULL;
 	// we keep a pointer to the packet
 	// this is extremely important because:
   // * binds[X].buffer does NOT point to a new allocated buffer
@@ -1533,13 +1460,6 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 			return NULL;
 		}
 		memcpy(&new_params_bound_flag,p+null_bitmap_length,1);
-		if (new_params_bound_flag==1) {
-			// something wrong
-//			delete ret;
-//			return NULL;
-			// the client is sending us the params type. We ignore it
-			//p+=(2*num_params);
-		}
 		uint8_t *null_bitmap=NULL;
 		null_bitmap=(uint8_t *)malloc(null_bitmap_length);
 		memcpy(null_bitmap,p,null_bitmap_length);
@@ -1579,7 +1499,6 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 			// the client is sending again the type of each parameter
 			for (i=0;i<num_params;i++) {
 				// set buffer_type and is_unsigned
-				//enum enum_field_types buffer_type=MYSQL_TYPE_DECIMAL; // set a random default
 				uint16_t buffer_type=0;
 				memcpy(&buffer_type,p,2);
 				binds[i].is_unsigned=0;
@@ -1593,23 +1512,8 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 				// set length, defaults to 0
 				// for parameters with not fixed length, that will be assigned later
 				lengths[i]=0;
-//			unsigned long l=0;
-//			uint8_t ll=mysql_decode_length((unsigned char *)p,&l);
-//			lengths[i]=l;
-//			p+=ll;
 				binds[i].length=&lengths[i];
-				//stmt_info->params[i]->buffer_type=binds[i].buffer_type;
 			}
-		} else { //new_params_bound_flag==0
-			// the client is NOT rebinding the parameters
-			// the client is NOT sending again the type of each parameter
-			// we should ALREADY know the type of each parameter
-			// nothing should be done here
-//			for (i=0;i<num_params;i++) {
-//				binds[i].buffer_type=stmt_info->params[i]->buffer_type;
-//				lengths[i]=0;
-//				binds[i].length=&lengths[i];
-//			}
 		}
 		for (i=0;i<num_params;i++) {
 			if (is_nulls[i]==true) {
@@ -1777,17 +1681,6 @@ MySQL_ResultSet::MySQL_ResultSet(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL
 	PSarrayOUT->add(pkt.ptr,pkt.size);
 	resultset_size+=pkt.size;
 	if (_stmt) { // binary protocol , we also assume we have ALL the resultset
-		//MYSQL_RES * prepare_meta_result = mysql_stmt_result_metadata(_stmt);
-//		int column_count=mysql_num_fields(prepare_meta_result);
-//		MYSQL_BIND *binds=(MYSQL_BIND *)malloc(sizeof(MYSQL_BIND)*column_count);
-//		mysql_stmt_bind_result(_stmt, binds);
-		//fprintf(stdout, "Fetching results ...\n");
-//		int row_count=0;
-//		while (!mysql_stmt_fetch(_stmt)) {
-//			row_count++;
-//			fprintf(stdout, "  row %d\n", row_count);
-//		}
-//		free (binds);
 		unsigned long long total_size=0;
 		MYSQL_ROWS *r=_stmt->result.data;
 		if (r) {
@@ -1831,8 +1724,6 @@ MySQL_ResultSet::MySQL_ResultSet(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL
 	}
 }
 
-
-
 MySQL_ResultSet::~MySQL_ResultSet() {
 	PtrSize_t pkt;
 	if (PSarrayOUT) {
