@@ -3,7 +3,7 @@
 #include "cpp.h"
 #include <string>
 #include <sys/utsname.h>
-
+#include "SpookyV2.h"
 
 static void term_handler(int sig) {
   proxy_warning("Received TERM signal: shutdown in progress...\n");
@@ -25,15 +25,9 @@ void crash_handler(int sig) {
 	fprintf(stderr, "Error: signal %d:\n", sig);
 	backtrace_symbols_fd(arr, s, STDERR_FILENO);
 #endif /* __GLIBC__ */
-//#ifdef SYS_gettid
 	// try to generate a core dump signaling again the thread
 	signal(sig, SIG_DFL);
-//	pid_t tid;
-//	tid = syscall(SYS_gettid);
 	pthread_kill(pthread_self(), sig);
-	//kill(tid, sig);
-//#endif /* SYS_gettid */
-//	exit(EXIT_FAILURE);
 }
 
 ProxySQL_GlobalVariables::~ProxySQL_GlobalVariables() {
@@ -54,23 +48,41 @@ ProxySQL_GlobalVariables::ProxySQL_GlobalVariables() {
 	__cmd_proxysql_initial=false;
 	__cmd_proxysql_reload=false;
 
+	statuses.stack_memory_mysql_threads = 0;
+	statuses.stack_memory_admin_threads = 0;
+	statuses.stack_memory_cluster_threads = 0;
+
+
 	global.gdbg=false;
 	global.nostart=false;
 	global.foreground=false;
 	global.monitor=true;
+#ifdef IDLE_THREADS
+	global.idle_threads=false;
+#endif /* IDLE_THREADS */
 #ifdef SO_REUSEPORT
-		global.reuseport=false;
+	global.reuseport=false;
 #endif /* SO_REUSEPORT */
 //	global.use_proxysql_mem=false;
 	pthread_mutex_init(&global.start_mutex,NULL);
+	pthread_mutex_init(&checksum_mutex,NULL);
+	epoch_version = 0;
+	checksums_values.updates_cnt = 0;
+	checksums_values.dumped_at = 0;
+	checksums_values.global_checksum = 0;
 #ifdef DEBUG
 	global.gdb=0;
 #endif
+
+	global.sqlite3_server=false;
+#ifdef PROXYSQLCLICKHOUSE
+	global.clickhouse_server=false;
+#endif /* PROXYSQLCLICKHOUSE */
 	opt=new ez::ezOptionParser();
 	opt->overview="High Performance Advanced Proxy for MySQL";
 	opt->syntax="proxysql [OPTIONS]";
 	std::string s = "\n\nProxySQL " ;
-	s = s + "rev. " + PROXYSQL_VERSION + " -- " + __TIMESTAMP__ + "\nCopyright (C) 2013-2016 René Cannaò\nThis program is free and without warranty\n";
+	s = s + "rev. " + PROXYSQL_VERSION + " -- " + __TIMESTAMP__ + "\nCopyright (C) 2013-2017 René Cannaò\nThis program is free and without warranty\n";
 	opt->footer =s.c_str();
 
 	opt->add((const char *)"",0,0,0,(const char *)"Display usage instructions.",(const char *)"-h",(const char *)"-help",(const char *)"--help",(const char *)"--usage");
@@ -86,12 +98,19 @@ ProxySQL_GlobalVariables::ProxySQL_GlobalVariables() {
 #endif /* SO_REUSEPORT */
 	opt->add((const char *)"",0,0,0,(const char *)"Do not restart ProxySQL if crashes",(const char *)"-e",(const char *)"--exit-on-error");
 	opt->add((const char *)"~/proxysql.cnf",0,1,0,(const char *)"Configuraton file",(const char *)"-c",(const char *)"--config");
-	//opt->add((const char *)"",0,0,0,(const char *)"Enable custom memory allocator",(const char *)"-m",(const char *)"--custom-memory");
 	opt->add((const char *)"",0,1,0,(const char *)"Datadir",(const char *)"-D",(const char *)"--datadir");
 	opt->add((const char *)"",0,0,0,(const char *)"Rename/empty database file",(const char *)"--initial");
 	opt->add((const char *)"",0,0,0,(const char *)"Merge config file into database file",(const char *)"--reload");
+#ifdef IDLE_THREADS
+	opt->add((const char *)"",0,0,0,(const char *)"Create auxiliary threads to handle idle connections",(const char *)"--idle-threads");
+#endif /* IDLE_THREADS */
 	opt->add((const char *)"",0,1,0,(const char *)"Administration Unix Socket",(const char *)"-S",(const char *)"--admin-socket");
 
+	opt->add((const char *)"",0,0,0,(const char *)"Enable SQLite3 Server",(const char *)"--sqlite3-server");
+#ifdef PROXYSQLCLICKHOUSE
+	opt->add((const char *)"",0,0,0,(const char *)"Enable ClickHouse Server",(const char *)"--clickhouse-server");
+#endif /* PROXYSQLCLICKHOUSE */
+
 	confFile=new ProxySQL_ConfigFile();
 };
 
@@ -107,8 +126,6 @@ void ProxySQL_GlobalVariables::parse(int argc, const char * argv[]) {
 };
 
 void ProxySQL_GlobalVariables::process_opts_pre() {
-
-
 	if (opt->isSet("-h")) {
 		std::string usage;
 		opt->getUsage(usage);
@@ -146,10 +163,6 @@ void ProxySQL_GlobalVariables::process_opts_pre() {
 		GloVars.__cmd_proxysql_datadir=strdup(datadir.c_str());
 	}
 
-//	if (opt->isSet("-m")) {
-//		global.use_proxysql_mem=true;
-//	}
-
 	if (opt->isSet("--initial")) {
 		__cmd_proxysql_initial=true;
 	}
@@ -158,7 +171,22 @@ void ProxySQL_GlobalVariables::process_opts_pre() {
 		__cmd_proxysql_reload=true;
 	}
 
-	
+#ifdef IDLE_THREADS
+	if (opt->isSet("--idle-threads")) {
+		global.idle_threads=true;
+	}
+#endif /* IDLE_THREADS */
+
+	if (opt->isSet("--sqlite3-server")) {
+		global.sqlite3_server=true;
+	}
+#ifdef PROXYSQLCLICKHOUSE
+	if (opt->isSet("--clickhouse-server")) {
+		global.clickhouse_server=true;
+	}
+#endif /* PROXYSQLCLICKHOUSE */
+
+
 	config_file=GloVars.__cmd_proxysql_config_file;
 
 	if (config_file==NULL) {
@@ -189,6 +217,11 @@ void ProxySQL_GlobalVariables::process_opts_post() {
 
 	if (opt->isSet("-f")) {
 		global.foreground=true;
+#ifdef __APPLE__
+	} else {
+		proxy_warning("ProxySQL does not support daemonize in Darwin: running in foreground\n");
+		global.foreground=true;
+#endif
 	}
 
 	if (opt->isSet("-M")) {
@@ -231,16 +264,11 @@ void ProxySQL_GlobalVariables::process_opts_post() {
 
 	proxy_debug(PROXY_DEBUG_GENERIC, 4, "processing opts\n");
 
-  //gchar *config_file=*config_file_ptr;
-
-
   // apply settings from cmdline, that have priority over config file
 #ifdef DEBUG
-//	if (GloVars.__cmd_proxysql_gdbg>0) { GloVars.global.gdbg=true; }
 	init_debug_struct_from_cmdline();
 #endif
 
-//	if (GloVars.__cmd_proxysql_foreground>=0) { foreground=GloVars.__cmd_proxysql_foreground; }
 	if (GloVars.__cmd_proxysql_nostart>=0) { glovars.nostart=GloVars.__cmd_proxysql_nostart; }
 	if (GloVars.__cmd_proxysql_datadir) {
 		free(glovars.proxy_datadir);
@@ -258,3 +286,45 @@ void ProxySQL_GlobalVariables::process_opts_post() {
   }
 #endif
 };
+
+
+uint64_t ProxySQL_GlobalVariables::generate_global_checksum() {
+	SpookyHash myhash;
+	myhash.Init(9,5);
+	ProxySQL_Checksum_Value *v = NULL;
+	v = &checksums_values.admin_variables;
+	if (v->version) {
+		myhash.Update(v->checksum,strlen(v->checksum));
+		myhash.Update(&v->version,sizeof(v->version));
+	}
+	v = &checksums_values.mysql_query_rules;
+	if (v->version) {
+		myhash.Update(v->checksum,strlen(v->checksum));
+		myhash.Update(&v->version,sizeof(v->version));
+	}
+	v = &checksums_values.mysql_servers;
+	if (v->version) {
+		myhash.Update(v->checksum,strlen(v->checksum));
+		myhash.Update(&v->version,sizeof(v->version));
+	}
+	v = &checksums_values.mysql_users;
+	if (v->version) {
+		myhash.Update(v->checksum,strlen(v->checksum));
+		myhash.Update(&v->version,sizeof(v->version));
+	}
+	v = &checksums_values.mysql_variables;
+	if (v->version) {
+		myhash.Update(v->checksum,strlen(v->checksum));
+		myhash.Update(&v->version,sizeof(v->version));
+	}
+	v = &checksums_values.proxysql_servers;
+	if (v->version) {
+		myhash.Update(v->checksum,strlen(v->checksum));
+		myhash.Update(&v->version,sizeof(v->version));
+	}
+	uint64_t h1, h2;
+	myhash.Final(&h1, &h2);
+	h1 = h1/2; // ugly way to make it signed within LLONG_MAX
+	checksums_values.global_checksum = h1;
+	return h1;
+}
