@@ -8,6 +8,8 @@
 
 #include "SpookyV2.h"
 
+#include "pcrecpp.h"
+
 #ifdef DEBUG
 #define DEB "_DEBUG"
 #else
@@ -15,6 +17,9 @@
 #endif /* DEBUG */
 #define QUERY_PROCESSOR_VERSION "0.2.0902" DEB
 
+#define QP_RE_MOD_CASELESS 1
+#define QP_RE_MOD_GLOBAL 2
+
 class QP_rule_text_hitsonly {
 	public:
 	char **pta;
@@ -37,7 +42,7 @@ class QP_rule_text {
 	char **pta;
 	int num_fields;
 	QP_rule_text(QP_rule_t *QPr) {
-		num_fields=27;
+		num_fields=32; // this count the number of fields
 		pta=NULL;
 		pta=(char **)malloc(sizeof(char *)*num_fields);
 		itostr(pta[0], (long long)QPr->rule_id);
@@ -51,9 +56,6 @@ class QP_rule_text {
 		itostr(pta[7], (long long)QPr->proxy_port);
 
 		char buf[20];
-		//uint32_t d32[2];
-		//memcpy(&d32,&QPr->digest,sizeof(QPr->digest));
-		//sprintf(buf,"0x%X%X", d32[0], d32[1]);
 		if (QPr->digest) {
 			sprintf(buf,"0x%016llX", (long long unsigned int)QPr->digest);
 			pta[8]=strdup(buf);
@@ -64,21 +66,35 @@ class QP_rule_text {
 		pta[9]=strdup_null(QPr->match_digest);
 		pta[10]=strdup_null(QPr->match_pattern);
 		itostr(pta[11], (long long)QPr->negate_match_pattern);
-		itostr(pta[12], (long long)QPr->flagOUT);
-		pta[13]=strdup_null(QPr->replace_pattern);
-		itostr(pta[14], (long long)QPr->destination_hostgroup);
-		itostr(pta[15], (long long)QPr->cache_ttl);
-		itostr(pta[16], (long long)QPr->reconnect);
-		itostr(pta[17], (long long)QPr->timeout);
-		itostr(pta[18], (long long)QPr->retries);
-		itostr(pta[19], (long long)QPr->delay);
-		itostr(pta[20], (long long)QPr->mirror_flagOUT);
-		itostr(pta[21], (long long)QPr->mirror_hostgroup);
-		pta[22]=strdup_null(QPr->error_msg);
-		itostr(pta[23], (long long)QPr->log);
-		itostr(pta[24], (long long)QPr->apply);
-		pta[25]=strdup_null(QPr->comment); // issue #643
-		itostr(pta[26], (long long)QPr->hits);
+		std::string re_mod;
+		re_mod="";
+		if ((QPr->re_modifiers & QP_RE_MOD_CASELESS) == QP_RE_MOD_CASELESS) re_mod = "CASELESS";
+			if ((QPr->re_modifiers & QP_RE_MOD_GLOBAL) == QP_RE_MOD_GLOBAL) {
+				if (re_mod.length()) {
+					re_mod = re_mod + ",";
+				}
+			re_mod = re_mod + "GLOBAL";
+		}
+		pta[12]=strdup_null((char *)re_mod.c_str()); // re_modifiers
+		itostr(pta[13], (long long)QPr->flagOUT);
+		pta[14]=strdup_null(QPr->replace_pattern);
+		itostr(pta[15], (long long)QPr->destination_hostgroup);
+		itostr(pta[16], (long long)QPr->cache_ttl);
+		itostr(pta[17], (long long)QPr->reconnect);
+		itostr(pta[18], (long long)QPr->timeout);
+		itostr(pta[19], (long long)QPr->retries);
+		itostr(pta[20], (long long)QPr->delay);
+		itostr(pta[21], (long long)QPr->next_query_flagIN);
+		itostr(pta[22], (long long)QPr->mirror_flagOUT);
+		itostr(pta[23], (long long)QPr->mirror_hostgroup);
+		pta[24]=strdup_null(QPr->error_msg);
+		pta[25]=strdup_null(QPr->OK_msg);
+		itostr(pta[26], (long long)QPr->sticky_conn);
+		itostr(pta[27], (long long)QPr->multiplex);
+		itostr(pta[28], (long long)QPr->log);
+		itostr(pta[29], (long long)QPr->apply);
+		pta[30]=strdup_null(QPr->comment); // issue #643
+		itostr(pta[31], (long long)QPr->hits);
 	}
 	~QP_rule_text() {
 		for(int i=0; i<num_fields; i++) {
@@ -88,17 +104,6 @@ class QP_rule_text {
 	}
 };
 
-/*
-struct __SQP_query_parser_t {
-	sfilter sf;
-	uint64_t digest;
-	char *digest_text;
-	char *first_comment;
-	uint64_t digest_total;
-};
-
-typedef struct __SQP_query_parser_t SQP_par_t;
-*/
 class QP_query_digest_stats {
 	public:
 	uint64_t digest;
@@ -161,9 +166,6 @@ class QP_query_digest_stats {
 		assert(username);
 		pta[1]=strdup(username);
 
-		//uint32_t d32[2];
-		//memcpy(&d32,&digest,sizeof(digest));
-		//sprintf(buf,"0x%X%X", d32[0], d32[1]);
 		sprintf(buf,"0x%016llX", (long long unsigned int)digest);
 		pta[2]=strdup(buf);
 
@@ -173,8 +175,6 @@ class QP_query_digest_stats {
 		pta[4]=strdup(buf);
 
 		time_t __now;
-    //char __buffer[25];
-//    struct tm *__tm_info;
     time(&__now);
 		
 		unsigned long long curtime=monotonic_time();
@@ -182,14 +182,10 @@ class QP_query_digest_stats {
 		time_t seen_time;
 
 		seen_time= __now - curtime/1000000 + first_seen/1000000;
-//    __tm_info = localtime(&seen_time);
-//    strftime(buf, 25, "%Y-%m-%d %H:%M:%S", __tm_info);
 		sprintf(buf,"%ld", seen_time);
 		pta[5]=strdup(buf);
 
 		seen_time= __now - curtime/1000000 + last_seen/1000000;
-//    __tm_info = localtime(&seen_time);
-//    strftime(buf, 25, "%Y-%m-%d %H:%M:%S", __tm_info);
 		sprintf(buf,"%ld", seen_time);
 		pta[6]=strdup(buf);
 
@@ -214,13 +210,11 @@ class QP_query_digest_stats {
 };
 
 
-//static char *commands_counters_desc[MYSQL_COM_QUERY___NONE];
-
-
-
 struct __RE2_objects_t {
-	re2::RE2::Options *opt;
-	RE2 *re;
+	pcrecpp::RE_Options *opt1;
+	pcrecpp::RE *re1;
+	re2::RE2::Options *opt2;
+	RE2 *re2;
 };
 
 typedef struct __RE2_objects_t re2_t;
@@ -229,12 +223,30 @@ static bool rules_sort_comp_function (QP_rule_t * a, QP_rule_t * b) { return (a-
 
 static re2_t * compile_query_rule(QP_rule_t *qr, int i) {
 	re2_t *r=(re2_t *)malloc(sizeof(re2_t));
-	r->opt=new re2::RE2::Options(RE2::Quiet);
-	r->opt->set_case_sensitive(false);
-	if (i==1) {
-		r->re=new RE2(qr->match_digest, *r->opt);
-	} else if (i==2) {
-		r->re=new RE2(qr->match_pattern, *r->opt);
+	r->opt1=NULL;
+	r->re1=NULL;
+	r->opt2=NULL;
+	r->re2=NULL;
+	if (mysql_thread___query_processor_regex==2) {
+		r->opt2=new re2::RE2::Options(RE2::Quiet);
+		if ((qr->re_modifiers & QP_RE_MOD_CASELESS) == QP_RE_MOD_CASELESS) {
+			r->opt2->set_case_sensitive(false);
+		}
+		if (i==1) {
+			r->re2=new RE2(qr->match_digest, *r->opt2);
+		} else if (i==2) {
+			r->re2=new RE2(qr->match_pattern, *r->opt2);
+		}
+	} else {
+		r->opt1=new pcrecpp::RE_Options();
+		if ((qr->re_modifiers & QP_RE_MOD_CASELESS) == QP_RE_MOD_CASELESS) {
+			r->opt1->set_caseless(true);
+		}
+		if (i==1) {
+			r->re1=new pcrecpp::RE(qr->match_digest, *r->opt1);
+		} else if (i==2) {
+			r->re1=new pcrecpp::RE(qr->match_pattern, *r->opt1);
+		}
 	}
 	return r;
 };
@@ -251,16 +263,22 @@ static void __delete_query_rule(QP_rule_t *qr) {
 		free(qr->replace_pattern);
 	if (qr->error_msg)
 		free(qr->error_msg);
+	if (qr->OK_msg)
+		free(qr->OK_msg);
 	if (qr->regex_engine1) {
 		re2_t *r=(re2_t *)qr->regex_engine1;
-		delete r->opt;
-		delete r->re;
+		if (r->opt1) { delete r->opt1; r->opt1=NULL; }
+		if (r->re1) { delete r->re1; r->re1=NULL; }
+		if (r->opt2) { delete r->opt2; r->opt2=NULL; }
+		if (r->re2) { delete r->re2; r->re2=NULL; }
 		free(qr->regex_engine1);
 	}
 	if (qr->regex_engine2) {
 		re2_t *r=(re2_t *)qr->regex_engine2;
-		delete r->opt;
-		delete r->re;
+		if (r->opt1) { delete r->opt1; r->opt1=NULL; }
+		if (r->re1) { delete r->re1; r->re1=NULL; }
+		if (r->opt2) { delete r->opt2; r->opt2=NULL; }
+		if (r->re2) { delete r->re2; r->re2=NULL; }
 		free(qr->regex_engine2);
 	}
 	free(qr);
@@ -283,7 +301,6 @@ static void __reset_rules(std::vector<QP_rule_t *> * qrs) {
 // per thread variables
 __thread unsigned int _thr_SQP_version;
 __thread std::vector<QP_rule_t *> * _thr_SQP_rules;
-//__thread unsigned int _thr_commands_counters[MYSQL_COM_QUERY___NONE];
 __thread Command_Counter * _thr_commands_counters[MYSQL_COM_QUERY___NONE];
 
 Query_Processor::Query_Processor() {
@@ -296,8 +313,13 @@ Query_Processor::Query_Processor() {
 		exit(EXIT_FAILURE);
 	}
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Initializing Query Processor with version=0\n");
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_init(&rwlock, NULL);
+	pthread_rwlock_init(&digest_rwlock, NULL);
+#else
 	spinlock_rwlock_init(&rwlock);
 	spinlock_rwlock_init(&digest_rwlock);
+#endif
 	version=0;
 	for (int i=0; i<MYSQL_COM_QUERY___NONE; i++) commands_counters[i]=new Command_Counter(i);
 
@@ -358,6 +380,11 @@ Query_Processor::Query_Processor() {
 Query_Processor::~Query_Processor() {
 	for (int i=0; i<MYSQL_COM_QUERY___NONE; i++) delete commands_counters[i];
 	__reset_rules(&rules);
+	for (std::unordered_map<uint64_t, void *>::iterator it=digest_umap.begin(); it!=digest_umap.end(); ++it) {
+		QP_query_digest_stats *qds=(QP_query_digest_stats *)it->second;
+		delete qds;
+	}
+	digest_umap.erase(digest_umap.begin(),digest_umap.end());
 };
 
 // This function is called by each thread when it starts. It create a Query Processor Table for each thread
@@ -381,16 +408,22 @@ void Query_Processor::print_version() {
 };
 
 void Query_Processor::wrlock() {
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&rwlock);
+#else
 	spin_wrlock(&rwlock);
+#endif
 };
 
 void Query_Processor::wrunlock() {
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&rwlock);
+#else
 	spin_wrunlock(&rwlock);
+#endif
 };
 
-
-
-QP_rule_t * Query_Processor::new_query_rule(int rule_id, bool active, char *username, char *schemaname, int flagIN, char *client_addr, char *proxy_addr, int proxy_port, char *digest, char *match_digest, char *match_pattern, bool negate_match_pattern, int flagOUT, char *replace_pattern, int destination_hostgroup, int cache_ttl, int reconnect, int timeout, int retries, int delay, int mirror_flagOUT, int mirror_hostgroup, char *error_msg, int log, bool apply, char *comment) {
+QP_rule_t * Query_Processor::new_query_rule(int rule_id, bool active, char *username, char *schemaname, int flagIN, char *client_addr, char *proxy_addr, int proxy_port, char *digest, char *match_digest, char *match_pattern, bool negate_match_pattern, char *re_modifiers, int flagOUT, char *replace_pattern, int destination_hostgroup, int cache_ttl, int reconnect, int timeout, int retries, int delay, int next_query_flagIN, int mirror_flagOUT, int mirror_hostgroup, char *error_msg, char *OK_msg, int sticky_conn, int multiplex, int log, bool apply, char *comment) {
 	QP_rule_t * newQR=(QP_rule_t *)malloc(sizeof(QP_rule_t));
 	newQR->rule_id=rule_id;
 	newQR->active=active;
@@ -400,6 +433,20 @@ QP_rule_t * Query_Processor::new_query_rule(int rule_id, bool active, char *user
 	newQR->match_digest=(match_digest ? strdup(match_digest) : NULL);
 	newQR->match_pattern=(match_pattern ? strdup(match_pattern) : NULL);
 	newQR->negate_match_pattern=negate_match_pattern;
+	newQR->re_modifiers=0;
+	{
+		tokenizer_t tok = tokenizer( re_modifiers, ",", TOKENIZER_NO_EMPTIES );
+		const char* token;
+		for (token = tokenize( &tok ); token; token = tokenize( &tok )) {
+			if (strncasecmp(token,(char *)"CASELESS",strlen((char *)"CASELESS"))==0) {
+				newQR->re_modifiers|=QP_RE_MOD_CASELESS;
+			}
+			if (strncasecmp(token,(char *)"GLOBAL",strlen((char *)"GLOBAL"))==0) {
+				newQR->re_modifiers|=QP_RE_MOD_GLOBAL;
+			}
+		}
+		free_tokenizer( &tok );
+	}
 	newQR->flagOUT=flagOUT;
 	newQR->replace_pattern=(replace_pattern ? strdup(replace_pattern) : NULL);
 	newQR->destination_hostgroup=destination_hostgroup;
@@ -408,9 +455,13 @@ QP_rule_t * Query_Processor::new_query_rule(int rule_id, bool active, char *user
 	newQR->timeout=timeout;
 	newQR->retries=retries;
 	newQR->delay=delay;
+	newQR->next_query_flagIN=next_query_flagIN;
 	newQR->mirror_flagOUT=mirror_flagOUT;
 	newQR->mirror_hostgroup=mirror_hostgroup;
 	newQR->error_msg=(error_msg ? strdup(error_msg) : NULL);
+	newQR->OK_msg=(OK_msg ? strdup(OK_msg) : NULL);
+	newQR->sticky_conn=sticky_conn;
+	newQR->multiplex=multiplex;
 	newQR->apply=apply;
 	newQR->comment=(comment ? strdup(comment) : NULL); // see issue #643
 	newQR->regex_engine1=NULL;
@@ -440,37 +491,73 @@ void Query_Processor::delete_query_rule(QP_rule_t *qr) {
 };
 
 void Query_Processor::reset_all(bool lock) {
-	if (lock) spin_wrlock(&rwlock);
+	if (lock)
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_wrlock(&rwlock);
+#else
+		spin_wrlock(&rwlock);
+#endif
 	__reset_rules(&rules);
-	if (lock) spin_wrunlock(&rwlock);
+	if (lock)
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_unlock(&rwlock);
+#else
+		spin_wrunlock(&rwlock);
+#endif
 };
 
 bool Query_Processor::insert(QP_rule_t *qr, bool lock) {
 	bool ret=true;
-	if (lock) spin_wrlock(&rwlock);
+	if (lock)
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_wrlock(&rwlock);
+#else
+		spin_wrlock(&rwlock);
+#endif
 	rules.push_back(qr);
-	if (lock) spin_wrunlock(&rwlock);
+	if (lock)
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_unlock(&rwlock);
+#else
+		spin_wrunlock(&rwlock);
+#endif
 	return ret;
 };
 
-
 void Query_Processor::sort(bool lock) {
-	if (lock) spin_wrlock(&rwlock);
+	if (lock)
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_wrlock(&rwlock);
+#else
+		spin_wrlock(&rwlock);
+#endif
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Sorting rules\n");
 	std::sort (rules.begin(), rules.end(), rules_sort_comp_function);
-	if (lock) spin_wrunlock(&rwlock);
+	if (lock)
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_unlock(&rwlock);
+#else
+		spin_wrunlock(&rwlock);
+#endif
 };
 
 // when commit is called, the version number is increased and the this will trigger the mysql threads to get a new Query Processor Table
 // The operation is asynchronous
 void Query_Processor::commit() {
-	spin_wrlock(&rwlock);
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_wrlock(&rwlock);
+#else
+		spin_wrlock(&rwlock);
+#endif
 	__sync_add_and_fetch(&version,1);
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Increasing version number to %d - all threads will notice this and refresh their rules\n", version);
-	spin_wrunlock(&rwlock);
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_unlock(&rwlock);
+#else
+		spin_wrunlock(&rwlock);
+#endif
 };
 
-
 SQLite3_result * Query_Processor::get_stats_commands_counters() {
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Dumping commands counters\n");
 	SQLite3_result *result=new SQLite3_result(15);
@@ -499,7 +586,11 @@ SQLite3_result * Query_Processor::get_stats_commands_counters() {
 SQLite3_result * Query_Processor::get_stats_query_rules() {
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Dumping query rules statistics, using Global version %d\n", version);
 	SQLite3_result *result=new SQLite3_result(2);
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&rwlock);
+#else
 	spin_rdlock(&rwlock);
+#endif
 	QP_rule_t *qr1;
 	result->add_column_definition(SQLITE_TEXT,"rule_id");
 	result->add_column_definition(SQLITE_TEXT,"hits");
@@ -512,14 +603,22 @@ SQLite3_result * Query_Processor::get_stats_query_rules() {
 			delete qt;
 		}
 	}
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&rwlock);
+#else
 	spin_rdunlock(&rwlock);
+#endif
 	return result;
 }
 
 SQLite3_result * Query_Processor::get_current_query_rules() {
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Dumping current query rules, using Global version %d\n", version);
-	SQLite3_result *result=new SQLite3_result(27);
+	SQLite3_result *result=new SQLite3_result(32);
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&rwlock);
+#else
 	spin_rdlock(&rwlock);
+#endif
 	QP_rule_t *qr1;
 	result->add_column_definition(SQLITE_TEXT,"rule_id");
 	result->add_column_definition(SQLITE_TEXT,"active");
@@ -533,6 +632,7 @@ SQLite3_result * Query_Processor::get_current_query_rules() {
 	result->add_column_definition(SQLITE_TEXT,"match_digest");
 	result->add_column_definition(SQLITE_TEXT,"match_pattern");
 	result->add_column_definition(SQLITE_TEXT,"negate_match_pattern");
+	result->add_column_definition(SQLITE_TEXT,"re_modifiers");
 	result->add_column_definition(SQLITE_TEXT,"flagOUT");
 	result->add_column_definition(SQLITE_TEXT,"replace_pattern");
 	result->add_column_definition(SQLITE_TEXT,"destination_hostgroup");
@@ -541,9 +641,13 @@ SQLite3_result * Query_Processor::get_current_query_rules() {
 	result->add_column_definition(SQLITE_TEXT,"timeout");
 	result->add_column_definition(SQLITE_TEXT,"retries");
 	result->add_column_definition(SQLITE_TEXT,"delay");
+	result->add_column_definition(SQLITE_TEXT,"next_query_flagIN");
 	result->add_column_definition(SQLITE_TEXT,"mirror_flagOUT");
 	result->add_column_definition(SQLITE_TEXT,"mirror_hostgroup");
 	result->add_column_definition(SQLITE_TEXT,"error_msg");
+	result->add_column_definition(SQLITE_TEXT,"OK_msg");
+	result->add_column_definition(SQLITE_TEXT,"sticky_conn");
+	result->add_column_definition(SQLITE_TEXT,"multiplex");
 	result->add_column_definition(SQLITE_TEXT,"log");
 	result->add_column_definition(SQLITE_TEXT,"apply");
 	result->add_column_definition(SQLITE_TEXT,"comment"); // issue #643
@@ -555,14 +659,53 @@ SQLite3_result * Query_Processor::get_current_query_rules() {
 		result->add_row(qt->pta);
 		delete qt;
 	}
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&rwlock);
+#else
 	spin_rdunlock(&rwlock);
+#endif
 	return result;
 }
 
+unsigned long Query_Processor::get_query_digests_total_size() {
+	unsigned long ret=0;
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&digest_rwlock);
+#else
+	spin_rdlock(&digest_rwlock);
+#endif
+	for (std::unordered_map<uint64_t, void *>::iterator it=digest_umap.begin(); it!=digest_umap.end(); ++it) {
+		ret += sizeof(QP_query_digest_stats);
+		QP_query_digest_stats *qds=(QP_query_digest_stats *)it->second;
+		if (qds->username)
+			ret += strlen(qds->username) + 1;
+		if (qds->schemaname)
+			ret += strlen(qds->schemaname) + 1;
+		if (qds->digest_text)
+			ret += strlen(qds->digest_text) + 1;
+	}
+#if !defined(__FreeBSD__) && !defined(__APPLE__)
+	ret += ((sizeof(uint64_t) + sizeof(void *) + sizeof(std::_Rb_tree_node_base)) * digest_umap.size() );
+#else
+	ret += ((sizeof(uint64_t) + sizeof(void *) + 32) * digest_umap.size() );
+#endif
+
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&digest_rwlock);
+#else
+	spin_rdunlock(&digest_rwlock);
+#endif
+	return ret;
+}
+
 SQLite3_result * Query_Processor::get_query_digests() {
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Dumping current query digest\n");
 	SQLite3_result *result=new SQLite3_result(11);
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&digest_rwlock);
+#else
 	spin_rdlock(&digest_rwlock);
+#endif
 	result->add_column_definition(SQLITE_TEXT,"hid");
 	result->add_column_definition(SQLITE_TEXT,"schemaname");
 	result->add_column_definition(SQLITE_TEXT,"usernname");
@@ -574,20 +717,27 @@ SQLite3_result * Query_Processor::get_query_digests() {
 	result->add_column_definition(SQLITE_TEXT,"sum_time");
 	result->add_column_definition(SQLITE_TEXT,"min_time");
 	result->add_column_definition(SQLITE_TEXT,"max_time");
-	//for (btree::btree_map<uint64_t, void *>::iterator it=digest_bt_map.begin(); it!=digest_bt_map.end(); ++it) {
 	for (std::unordered_map<uint64_t, void *>::iterator it=digest_umap.begin(); it!=digest_umap.end(); ++it) {
 		QP_query_digest_stats *qds=(QP_query_digest_stats *)it->second;
 		char **pta=qds->get_row();
 		result->add_row(pta);
 		qds->free_row(pta);
 	}
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&digest_rwlock);
+#else
 	spin_rdunlock(&digest_rwlock);
+#endif
 	return result;
 }
 
 SQLite3_result * Query_Processor::get_query_digests_reset() {
 	SQLite3_result *result=new SQLite3_result(11);
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&digest_rwlock);
+#else
 	spin_wrlock(&digest_rwlock);
+#endif
 	result->add_column_definition(SQLITE_TEXT,"hid");
 	result->add_column_definition(SQLITE_TEXT,"schemaname");
 	result->add_column_definition(SQLITE_TEXT,"usernname");
@@ -599,7 +749,6 @@ SQLite3_result * Query_Processor::get_query_digests_reset() {
 	result->add_column_definition(SQLITE_TEXT,"sum_time");
 	result->add_column_definition(SQLITE_TEXT,"min_time");
 	result->add_column_definition(SQLITE_TEXT,"max_time");
-	//for (btree::btree_map<uint64_t, void *>::iterator it=digest_bt_map.begin(); it!=digest_bt_map.end(); ++it) {
 	for (std::unordered_map<uint64_t, void *>::iterator it=digest_umap.begin(); it!=digest_umap.end(); ++it) {
 		QP_query_digest_stats *qds=(QP_query_digest_stats *)it->second;
 		char **pta=qds->get_row();
@@ -607,9 +756,12 @@ SQLite3_result * Query_Processor::get_query_digests_reset() {
 		qds->free_row(pta);
 		delete qds;
 	}
-	//digest_bt_map.erase(digest_bt_map.begin(),digest_bt_map.end());
 	digest_umap.erase(digest_umap.begin(),digest_umap.end());
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&digest_rwlock);
+#else
 	spin_wrunlock(&digest_rwlock);
+#endif
 	return result;
 }
 
@@ -617,22 +769,31 @@ SQLite3_result * Query_Processor::get_query_digests_reset() {
 
 Query_Processor_Output * Query_Processor::process_mysql_query(MySQL_Session *sess, void *ptr, unsigned int size, Query_Info *qi) {
 	// to avoid unnecssary deallocation/allocation, we initialize qpo witout new allocation
-	//Query_Processor_Output *ret=NULL;
-	//ret=new Query_Processor_Output();
 	Query_Processor_Output *ret=sess->qpo;
 	ret->init();
 	SQP_par_t *qp=NULL;
 	if (qi) {
 		qp=(SQP_par_t *)&qi->QueryParserArgs;
 	}
+#define stackbuffer_size 128
+	char stackbuffer[stackbuffer_size];
 	unsigned int len=size-sizeof(mysql_hdr)-1;
-	char *query=(char *)l_alloc(len+1);
+	char *query=NULL;
+	if (len < stackbuffer_size) {
+		query=stackbuffer;
+	} else {
+		query=(char *)l_alloc(len+1);
+	}
 	memcpy(query,(char *)ptr+sizeof(mysql_hdr)+1,len);
 	query[len]=0;
 	if (__sync_add_and_fetch(&version,0) > _thr_SQP_version) {
 		// update local rules;
 		proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Detected a changed in version. Global:%d , local:%d . Refreshing...\n", version, _thr_SQP_version);
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_rdlock(&rwlock);
+#else
 		spin_rdlock(&rwlock);
+#endif
 		_thr_SQP_version=__sync_add_and_fetch(&version,0);
 		__reset_rules(_thr_SQP_rules);
 		QP_rule_t *qr1;
@@ -643,18 +804,24 @@ Query_Processor_Output * Query_Processor::process_mysql_query(MySQL_Session *ses
 				proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Copying Query Rule id: %d\n", qr1->rule_id);
 				char buf[20];
 				if (qr1->digest) { // not 0
-					//uint32_t d32[2];
-					//memcpy(&d32,&qr1->digest,sizeof(qr1->digest));
-					//sprintf(buf,"0x%X%X", d32[0], d32[1]);
 					sprintf(buf,"0x%016llX", (long long unsigned int)qr1->digest);
 				}
+				std::string re_mod;
+				re_mod="";
+				if ((qr1->re_modifiers & QP_RE_MOD_CASELESS) == QP_RE_MOD_CASELESS) re_mod = "CASELESS";
+				if ((qr1->re_modifiers & QP_RE_MOD_GLOBAL) == QP_RE_MOD_GLOBAL) {
+					if (re_mod.length()) {
+						re_mod = re_mod + ",";
+					}
+					re_mod = re_mod + "GLOBAL";
+				}
 				qr2=new_query_rule(qr1->rule_id, qr1->active, qr1->username, qr1->schemaname, qr1->flagIN,
 					qr1->client_addr, qr1->proxy_addr, qr1->proxy_port,
 					( qr1->digest ? buf : NULL ) ,
-					qr1->match_digest, qr1->match_pattern, qr1->negate_match_pattern,
+					qr1->match_digest, qr1->match_pattern, qr1->negate_match_pattern, (char *)re_mod.c_str(),
 					qr1->flagOUT, qr1->replace_pattern, qr1->destination_hostgroup,
-					qr1->cache_ttl, qr1->reconnect, qr1->timeout, qr1->retries, qr1->delay, qr1->mirror_flagOUT, qr1->mirror_hostgroup,
-					qr1->error_msg, qr1->log, qr1->apply,
+					qr1->cache_ttl, qr1->reconnect, qr1->timeout, qr1->retries, qr1->delay, qr1->next_query_flagIN, qr1->mirror_flagOUT, qr1->mirror_hostgroup,
+					qr1->error_msg, qr1->OK_msg, qr1->sticky_conn, qr1->multiplex, qr1->log, qr1->apply,
 					qr1->comment);
 				qr2->parent=qr1;	// pointer to parent to speed up parent update (hits)
 				if (qr2->match_digest) {
@@ -668,11 +835,19 @@ Query_Processor_Output * Query_Processor::process_mysql_query(MySQL_Session *ses
 				_thr_SQP_rules->push_back(qr2);
 			}
 		}
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+		pthread_rwlock_unlock(&rwlock);
+#else
 		spin_rdunlock(&rwlock); // unlock should be after the copy
+#endif
 	}
 	QP_rule_t *qr;
 	re2_t *re2p;
 	int flagIN=0;
+	ret->next_query_flagIN=-1; // reset
+	if (sess->next_query_flagIN >= 0) {
+		flagIN=sess->next_query_flagIN;
+	}
 	int reiterate=mysql_thread___query_processor_iterations;
 	if (sess->mirror==true) {
 		// we are into a mirror session
@@ -752,7 +927,11 @@ __internal_loop:
 			if (qr->match_digest) {
 				bool rc;
 				// we always match on original query
-				rc=RE2::PartialMatch(qp->digest_text,*re2p->re);
+				if (re2p->re2) {
+					rc=RE2::PartialMatch(qp->digest_text,*re2p->re2);
+				} else {
+					rc=re2p->re1->PartialMatch(qp->digest_text);
+				}
 				if ((rc==true && qr->negate_match_pattern==true) || ( rc==false && qr->negate_match_pattern==false )) {
 					proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has no matching pattern\n", qr->rule_id);
 					continue;
@@ -766,10 +945,18 @@ __internal_loop:
 			if (ret && ret->new_query) {
 				// if we already rewrote the query, process the new query
 				//std::string *s=ret->new_query;
-				rc=RE2::PartialMatch(ret->new_query->c_str(),*re2p->re);
+				if (re2p->re2) {
+					rc=RE2::PartialMatch(ret->new_query->c_str(),*re2p->re2);
+				} else {
+					rc=re2p->re1->PartialMatch(ret->new_query->c_str());
+				}
 			} else {
 				// we never rewrote the query
-				rc=RE2::PartialMatch(query,*re2p->re);
+				if (re2p->re2) {
+					rc=RE2::PartialMatch(query,*re2p->re2);
+				} else {
+					rc=re2p->re1->PartialMatch(query);
+				}
 			}
 			if ((rc==true && qr->negate_match_pattern==true) || ( rc==false && qr->negate_match_pattern==false )) {
 				proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has no matching pattern\n", qr->rule_id);
@@ -806,6 +993,11 @@ __internal_loop:
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set delay: %d. Session will%s be paused for %dms\n", qr->rule_id, qr->delay, (qr->delay == 0 ? " NOT" : "" ) , qr->delay);
       ret->delay=qr->delay;
     }
+    if (qr->next_query_flagIN >= 0) {
+			// Note: Negative next_query_flagIN means this rule doesn't change the next query flagIN
+        proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set next query flagIN: %d\n", qr->rule_id, qr->next_query_flagIN);
+        ret->next_query_flagIN=qr->next_query_flagIN;
+    }
     if (qr->mirror_flagOUT >= 0) {
 			// Note: negative mirror_flagOUT means this rule doesn't change the mirror flagOUT
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set mirror flagOUT: %d\n", qr->rule_id, qr->mirror_flagOUT);
@@ -821,11 +1013,26 @@ __internal_loop:
 			//proxy_warning("User \"%s\" has issued query that has been filtered: %s \n " , sess->client_myds->myconn->userinfo->username, query);
       ret->error_msg=strdup(qr->error_msg);
     }
+    if (qr->OK_msg) {
+      proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set error_msg: %s\n", qr->rule_id, qr->OK_msg);
+			//proxy_warning("User \"%s\" has issued query that has been filtered: %s \n " , sess->client_myds->myconn->userinfo->username, query);
+      ret->OK_msg=strdup(qr->OK_msg);
+    }
     if (qr->cache_ttl >= 0) {
 			// Note: negative TTL means this rule doesn't change 
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set cache_ttl: %d. Query will%s hit the cache\n", qr->rule_id, qr->cache_ttl, (qr->cache_ttl == 0 ? " NOT" : "" ));
       ret->cache_ttl=qr->cache_ttl;
     }
+    if (qr->sticky_conn >= 0) {
+			// Note: negative sticky_conn means this rule doesn't change
+      proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set sticky_conn: %d. Connection will%s stick\n", qr->rule_id, qr->sticky_conn, (qr->sticky_conn == 0 ? " NOT" : "" ));
+      ret->sticky_conn=qr->sticky_conn;
+    }
+    if (qr->multiplex >= 0) {
+			// Note: negative multiplex means this rule doesn't change
+      proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set multiplex: %d. Connection will%s multiplex\n", qr->rule_id, qr->multiplex, (qr->multiplex == 0 ? " NOT" : "" ));
+      ret->multiplex=qr->multiplex;
+    }
     if (qr->log >= 0) {
 			// Note: negative log means this rule doesn't change
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set log: %d. Query will%s logged\n", qr->rule_id, qr->log, (qr->log == 0 ? " NOT" : "" ));
@@ -840,7 +1047,22 @@ __internal_loop:
 		if (qr->replace_pattern) {
 			proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d on match_pattern \"%s\" has a replace_pattern \"%s\" to apply\n", qr->rule_id, qr->match_pattern, qr->replace_pattern);
 			if (ret->new_query==NULL) ret->new_query=new std::string(query);
-			RE2::Replace(ret->new_query,qr->match_pattern,qr->replace_pattern);
+			re2_t *re2p=(re2_t *)qr->regex_engine2;
+			if (re2p->re2) {
+				//RE2::Replace(ret->new_query,qr->match_pattern,qr->replace_pattern);
+				if ((qr->re_modifiers & QP_RE_MOD_GLOBAL) == QP_RE_MOD_GLOBAL) {
+					re2p->re2->GlobalReplace(ret->new_query,qr->match_pattern,qr->replace_pattern);
+				} else {
+					re2p->re2->Replace(ret->new_query,qr->match_pattern,qr->replace_pattern);
+				}
+			} else {
+				//re2p->re1->Replace(ret->new_query,qr->replace_pattern);
+				if ((qr->re_modifiers & QP_RE_MOD_GLOBAL) == QP_RE_MOD_GLOBAL) {
+					re2p->re1->GlobalReplace(qr->replace_pattern,ret->new_query);
+				} else {
+					re2p->re1->Replace(qr->replace_pattern,ret->new_query);
+				}
+			}
 		}
 
 		if (qr->apply==true) {
@@ -857,7 +1079,11 @@ __internal_loop:
 	
 __exit_process_mysql_query:
 	// FIXME : there is too much data being copied around
-	l_free(len+1,query);
+	if (len < stackbuffer_size) {
+		// query is in the stack
+	} else {
+		l_free(len+1,query);
+	}
 	if (sess->mirror==false) { // we process comments only on original queries, not on mirrors
 		if (qp && qp->first_comment) {
 			// we have a comment to parse
@@ -886,7 +1112,11 @@ void Query_Processor::update_query_processor_stats() {
 	// It acquires a read lock to ensure that the rules table doesn't change
 	// Yet, because it has to update vales, it uses atomic operations
 	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "Updating query rules statistics\n");
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_rdlock(&rwlock);
+#else
 	spin_rdlock(&rwlock);
+#endif
 	if (__sync_add_and_fetch(&version,0) == _thr_SQP_version) {
 		QP_rule_t *qr;
 		for (std::vector<QP_rule_t *>::iterator it=_thr_SQP_rules->begin(); it!=_thr_SQP_rules->end(); ++it) {
@@ -897,7 +1127,11 @@ void Query_Processor::update_query_processor_stats() {
 			}
 		}
 	}
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&rwlock);
+#else
 	spin_rdunlock(&rwlock);
+#endif
 	for (int i=0; i<MYSQL_COM_QUERY___NONE; i++) {
 		for (int j=0; j<13; j++) {
 			if (_thr_commands_counters[i]->counters[j]) {
@@ -915,15 +1149,11 @@ void Query_Processor::update_query_processor_stats() {
 void Query_Processor::query_parser_init(SQP_par_t *qp, char *query, int query_length, int flags) {
 	// trying to get rid of libinjection
 	// instead of initializing qp->sf , we copy query info later in this function
-//	if (mysql_thread___commands_stats)
-//		libinjection_sqli_init(&qp->sf, query, query_length, FLAG_SQL_MYSQL);
 	qp->digest_text=NULL;
 	qp->first_comment=NULL;
 	qp->query_prefix=NULL;
-	//qp->first_comment=(char *)l_alloc(FIRST_COMMENT_MAX_LENGTH);
-	//qp->first_comment[0]=0; // initialize it to 0 . Useful to determine if there is any string or not
 	if (mysql_thread___query_digests) {
-		qp->digest_text=mysql_query_digest_and_first_comment(query, query_length, &qp->first_comment);
+		qp->digest_text=mysql_query_digest_and_first_comment(query, query_length, &qp->first_comment, ((query_length < QUERY_DIGEST_BUF) ? qp->buf : NULL));
 		// the hash is computed only up to query_digests_max_digest_length bytes
 		int digest_text_length=strnlen(qp->digest_text, mysql_thread___query_digests_max_digest_length);
 		qp->digest=SpookyHash::Hash64(qp->digest_text, digest_text_length, 0);
@@ -933,7 +1163,6 @@ void Query_Processor::query_parser_init(SQP_par_t *qp, char *query, int query_le
 		}
 #endif /* DEBUG */
 	} else {
-		// if mysql_thread___query_digests==false but we still want command statistics, we copy the prefix of the query
 		if (mysql_thread___commands_stats) {
 			size_t sl=32;
 			if ((unsigned int)query_length < sl) {
@@ -997,8 +1226,11 @@ unsigned long long Query_Processor::query_parser_update_counters(MySQL_Session *
 }
 
 void Query_Processor::update_query_digest(SQP_par_t *qp, int hid, MySQL_Connection_userinfo *ui, unsigned long long t, unsigned long long n, MySQL_STMT_Global_info *_stmt_info) {
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_wrlock(&digest_rwlock);
+#else
 	spin_wrlock(&digest_rwlock);
-
+#endif
 	QP_query_digest_stats *qds;	
 
 	std::unordered_map<uint64_t, void *>::iterator it;
@@ -1017,23 +1249,24 @@ void Query_Processor::update_query_digest(SQP_par_t *qp, int hid, MySQL_Connecti
 		digest_umap.insert(std::make_pair(qp->digest_total,(void *)qds));
 	}
 
+#ifdef PROXYSQL_QPRO_PTHREAD_MUTEX
+	pthread_rwlock_unlock(&digest_rwlock);
+#else
 	spin_wrunlock(&digest_rwlock);
+#endif
 }
 
 char * Query_Processor::get_digest_text(SQP_par_t *qp) {
 	if (qp==NULL) return NULL;
-	//SQP_par_t *qp=(SQP_par_t *)p;
 	return qp->digest_text;
 }
 
 uint64_t Query_Processor::get_digest(SQP_par_t *qp) {
 	if (qp==NULL) return 0;
-	//SQP_par_t *qp=(SQP_par_t *)args;
 	return qp->digest;
 }
 
 enum MYSQL_COM_QUERY_command Query_Processor::__query_parser_command_type(SQP_par_t *qp) {
-	//SQP_par_t *qp=(SQP_par_t *)args;
 	char *text=NULL; // this new variable is a pointer to either qp->digest_text , or to the query
 	if (qp->digest_text) {
 		text=qp->digest_text;
@@ -1041,7 +1274,6 @@ enum MYSQL_COM_QUERY_command Query_Processor::__query_parser_command_type(SQP_pa
 		text=qp->query_prefix;
 	}
 
-
 	enum MYSQL_COM_QUERY_command ret=MYSQL_COM_QUERY_UNKNOWN;
 	char c1;
 
@@ -1052,7 +1284,6 @@ enum MYSQL_COM_QUERY_command Query_Processor::__query_parser_command_type(SQP_pa
 		goto __exit__query_parser_command_type;
 	}
 
-	//c1=toupper(token[0]);
 	c1=token[0];
 	proxy_debug(PROXY_DEBUG_MYSQL_COM, 5, "Command:%s Prefix:%c\n", token, c1);
 	switch (c1) {
@@ -1446,10 +1677,11 @@ bool Query_Processor::query_parser_first_comment(Query_Processor_Output *qpo, ch
 	return ret;
 }
 
-
 void Query_Processor::query_parser_free(SQP_par_t *qp) {
 	if (qp->digest_text) {
-		free(qp->digest_text);
+		if (qp->digest_text != qp->buf) {
+			free(qp->digest_text);
+		}
 		qp->digest_text=NULL;
 	}
 	if (qp->first_comment) {
