@@ -1,27 +1,12 @@
 #include "proxysql.h"
 #include "cpp.h"
 #include "SpookyV2.h"
+#include <fcntl.h>
 
 extern const CHARSET_INFO * proxysql_find_charset_nr(unsigned int nr);
 
 #define PROXYSQL_USE_RESULT
 
-// Bug https://mariadb.atlassian.net/browse/CONC-136
-//int STDCALL mysql_select_db_start(int *ret, MYSQL *mysql, const char *db);
-//int STDCALL mysql_select_db_cont(int *ret, MYSQL *mysql, int ready_status);
-
-/*
-void * MySQL_Connection::operator new(size_t size) {
-	return l_alloc(size);
-}
-
-void MySQL_Connection::operator delete(void *ptr) {
-	l_free(sizeof(MySQL_Connection),ptr);
-}
-*/
-
-//extern __thread char *mysql_thread___default_schema;
-
 static int
 mysql_status(short event, short cont) {
 	int status= 0;
@@ -45,7 +30,6 @@ MySQL_Connection_userinfo::MySQL_Connection_userinfo() {
 	sha1_pass=NULL;
 	schemaname=NULL;
 	hash=0;
-	//schemaname=strdup(mysql_thread___default_schema);
 }
 
 MySQL_Connection_userinfo::~MySQL_Connection_userinfo() {
@@ -93,12 +77,24 @@ uint64_t MySQL_Connection_userinfo::compute_hash() {
 
 void MySQL_Connection_userinfo::set(char *u, char *p, char *s, char *sh1) {
 	if (u) {
-		if (username) free(username);
-		username=strdup(u);
+		if (username) {
+			if (strcmp(u,username)) {
+				free(username);
+				username=strdup(u);
+			}
+		} else {
+			username=strdup(u);
+		}
 	}
 	if (p) {
-		if (password) free(password);
-		password=strdup(p);
+		if (password) {
+			if (strcmp(p,password)) {
+				free(password);
+				password=strdup(p);
+			}
+		} else {
+			password=strdup(p);
+		}
 	}
 	if (s) {
 		if (schemaname) free(schemaname);
@@ -147,7 +143,6 @@ bool MySQL_Connection_userinfo::set_schemaname(char *_new, int l) {
 
 
 MySQL_Connection::MySQL_Connection() {
-	//memset(&myconn,0,sizeof(MYSQL));
 	mysql=NULL;
 	async_state_machine=ASYNC_CONNECT_START;
 	ret_mysql=NULL;
@@ -181,11 +176,16 @@ MySQL_Connection::MySQL_Connection() {
 	query.stmt_meta=NULL;
 	query.stmt_result=NULL;
 	largest_query_length=0;
+	multiplex_delayed=false;
 	MyRS=NULL;
 	creation_time=0;
 	processing_multi_statement=false;
 	proxy_debug(PROXY_DEBUG_MYSQL_CONNPOOL, 4, "Creating new MySQL_Connection %p\n", this);
+#ifndef PROXYSQL_STMT_V14
 	local_stmts=new MySQL_STMTs_local(false); // false by default, it is a backend
+#else
+	local_stmts=new MySQL_STMTs_local_v14(false); // false by default, it is a backend
+#endif
 };
 
 MySQL_Connection::~MySQL_Connection() {
@@ -204,15 +204,6 @@ MySQL_Connection::~MySQL_Connection() {
 		close_mysql(); // this take care of closing mysql connection
 		mysql=NULL;
 	}
-//	// FIXME: with the use of mysql client library , this part should be gone.
-//	// for now only commenting it to be sure it is not needed 
-//	if (myds) {
-//		myds->shut_hard();
-//	} else {
-//		proxy_debug(PROXY_DEBUG_MYSQL_CONNPOOL, 4, "MySQL_Connection %p , fd:%d\n", this, fd);
-//		shutdown(fd, SHUT_RDWR);
-//		close(fd);
-//	}
 	if (MyRS) {
 		delete MyRS;
 	}
@@ -220,8 +211,6 @@ MySQL_Connection::~MySQL_Connection() {
 		delete local_stmts;
 	}
 	if (query.stmt) {
-		// we don't run mysql_stmt_close() : should be already destroyed
-//		mysql_stmt_close(query.stmt);
 		query.stmt=NULL;
 	}
 	if (options.sql_mode) {
@@ -316,6 +305,14 @@ void MySQL_Connection::set_status_prepared_statement(bool v) {
 	}
 }
 
+void MySQL_Connection::set_status_no_multiplex(bool v) {
+	if (v) {
+		status_flags |= STATUS_MYSQL_CONNECTION_NO_MULTIPLEX;
+	} else {
+		status_flags &= ~STATUS_MYSQL_CONNECTION_NO_MULTIPLEX;
+	}
+}
+
 // pay attention here. set_status_sql_log_bin0 sets it sql_log_bin is ZERO
 // sql_log_bin=0 => true
 // sql_log_bin=1 => false
@@ -359,6 +356,10 @@ bool MySQL_Connection::get_status_prepared_statement() {
 	return status_flags & STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT;
 }
 
+bool MySQL_Connection::get_status_no_multiplex() {
+	return status_flags & STATUS_MYSQL_CONNECTION_NO_MULTIPLEX;
+}
+
 bool MySQL_Connection::get_status_sql_log_bin0() {
 	return status_flags & STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0;
 }
@@ -485,11 +486,8 @@ void MySQL_Connection::set_query(char *stmt, unsigned long length) {
 		largest_query_length=length;
 	}
 	if (query.stmt) {
-		//mysql_stmt_close(query.stmt);
 		query.stmt=NULL;
 	}
-	//query.ptr=(char *)malloc(length);
-	//memcpy(query.ptr,stmt,length);
 }
 
 void MySQL_Connection::real_query_start() {
@@ -766,8 +764,6 @@ handler_again:
 			__sync_fetch_and_add(&parent->queries_sent,1);
 			__sync_fetch_and_add(&parent->bytes_sent,query.stmt_meta->size);
 			myds->sess->thread->status_variables.queries_backends_bytes_sent+=query.stmt_meta->size;
-//			__sync_fetch_and_add(&parent->bytes_sent,query.length);
-//			myds->sess->thread->status_variables.queries_backends_bytes_sent+=query.length;
 			if (async_exit_status) {
 				next_event(ASYNC_STMT_EXECUTE_CONT);
 			} else {
@@ -831,15 +827,6 @@ handler_again:
 					__sync_fetch_and_add(&parent->bytes_recv,total_size);
 					myds->sess->thread->status_variables.queries_backends_bytes_recv+=total_size;
 				}
-/*
-				int row_count= 0;
-				fprintf(stdout, "Fetching results ...\n");
-				while (!mysql_stmt_fetch(query.stmt))
-				{
-					row_count++;
-					fprintf(stdout, "  row %d\n", row_count);
-				}
-*/
 			}
 /*
 			if (interr) {
@@ -1138,6 +1125,22 @@ int MySQL_Connection::async_connect(short event) {
 }
 
 
+bool MySQL_Connection::IsServerOffline() {
+	bool ret=false;
+	if (parent==NULL)
+		return ret;
+	server_status=parent->status; // we copy it here to avoid race condition. The caller will see this
+	if (
+		(server_status==MYSQL_SERVER_STATUS_OFFLINE_HARD) // the server is OFFLINE as specific by the user
+		||
+		(server_status==MYSQL_SERVER_STATUS_SHUNNED && parent->shunned_automatic==true && parent->shunned_and_kill_all_connections==true) // the server is SHUNNED due to a serious issue
+		||
+		(server_status==MYSQL_SERVER_STATUS_SHUNNED_REPLICATION_LAG) // slave is lagging! see #774
+	) {
+		ret=true;
+	}
+	return ret;
+}
 
 // Returns:
 // 0 when the query is completed
@@ -1201,8 +1204,6 @@ int MySQL_Connection::async_query(short event, char *stmt, unsigned long length,
 	if (async_state_machine==ASYNC_STMT_PREPARE_SUCCESSFUL || async_state_machine==ASYNC_STMT_PREPARE_FAILED) {
 		query.stmt_meta=NULL;
 		if (async_state_machine==ASYNC_STMT_PREPARE_FAILED) {
-			//mysql_stmt_close(query.stmt);
-			//query.stmt=NULL;
 			return -1;
 		} else {
 			*_stmt=query.stmt;
@@ -1417,23 +1418,30 @@ void MySQL_Connection::async_free_result() {
 	//assert(ret_mysql);
 	//assert(async_state_machine==ASYNC_QUERY_END);
 	if (query.ptr) {
-		//free(query.ptr);
 		query.ptr=NULL;
 		query.length=0;
 	}
-	if (query.stmt_result) {
-		mysql_free_result(query.stmt_result);
-		query.stmt_result=NULL;
-	}
-	if (query.stmt) {
-		if (query.stmt->mysql) {
-			mysql_stmt_free_result(query.stmt);
+	if (userinfo) {
+		// if userinfo is NULL , the connection is being destroyed
+		// because it is reset on destructor ( ~MySQL_Connection() )
+		// therefore this section is skipped completely
+		// this should prevent bug #1046
+		if (query.stmt_result) {
+			mysql_free_result(query.stmt_result);
+			query.stmt_result=NULL;
+		}
+		if (query.stmt) {
+			if (query.stmt->mysql) {
+				if (query.stmt->mysql == mysql) { // extra check
+					mysql_stmt_free_result(query.stmt);
+				}
+			}
+			query.stmt=NULL;
+		}
+		if (mysql_result) {
+			mysql_free_result(mysql_result);
+			mysql_result=NULL;
 		}
-		query.stmt=NULL;
-	}
-	if (mysql_result) {
-		mysql_free_result(mysql_result);
-		mysql_result=NULL;
 	}
 	async_state_machine=ASYNC_IDLE;
 	if (MyRS) {
@@ -1442,6 +1450,7 @@ void MySQL_Connection::async_free_result() {
 	}
 }
 
+/*
 bool MySQL_Connection::IsActiveTransaction() {
 	bool ret=false;
 	if (mysql) {
@@ -1452,6 +1461,7 @@ bool MySQL_Connection::IsActiveTransaction() {
 	}
 	return ret;
 }
+*/
 
 bool MySQL_Connection::IsAutoCommit() {
 	bool ret=false;
@@ -1473,7 +1483,7 @@ bool MySQL_Connection::MultiplexDisabled() {
 // status_flags stores information about the status of the connection
 // can be used to determine if multiplexing can be enabled or not
 	bool ret=false;
-	if (status_flags & (STATUS_MYSQL_CONNECTION_TRANSACTION|STATUS_MYSQL_CONNECTION_USER_VARIABLE|STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT|STATUS_MYSQL_CONNECTION_LOCK_TABLES|STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE|STATUS_MYSQL_CONNECTION_GET_LOCK|STATUS_MYSQL_CONNECTION_FOUND_ROWS|STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0) ) {
+	if (status_flags & (STATUS_MYSQL_CONNECTION_TRANSACTION|STATUS_MYSQL_CONNECTION_USER_VARIABLE|STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT|STATUS_MYSQL_CONNECTION_LOCK_TABLES|STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE|STATUS_MYSQL_CONNECTION_GET_LOCK|STATUS_MYSQL_CONNECTION_NO_MULTIPLEX|STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0|STATUS_MYSQL_CONNECTION_FOUND_ROWS) ) {
 		ret=true;
 	}
 	return ret;
@@ -1482,8 +1492,24 @@ bool MySQL_Connection::MultiplexDisabled() {
 
 void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 	if (query_digest_text==NULL) return;
+	// unknown what to do with multiplex
+	int mul=-1;
+	if (myds) {
+		if (myds->sess) {
+			if (myds->sess->qpo) {
+				mul=myds->sess->qpo->multiplex;
+				if (mul==0) {
+					set_status_no_multiplex(true);
+				} else {
+					if (mul==1) {
+						set_status_no_multiplex(false);
+					}
+				}
+			}
+		}
+	}
 	if (get_status_user_variable()==false) { // we search for variables only if not already set
-		if (index(query_digest_text,'@')) {
+		if (mul!=2 && index(query_digest_text,'@')) { // mul = 2 has a special meaning : do not disable multiplex for variables in THIS QUERY ONLY
 			if (
 				strncasecmp(query_digest_text,"SELECT @@tx_isolation", strlen("SELECT @@tx_isolation"))
 				&&
@@ -1566,20 +1592,16 @@ void MySQL_Connection::close_mysql() {
 		memcpy(buff, &myhdr, sizeof(mysql_hdr));
 		buff[4]=0x01;
 		int fd=mysql->net.fd;
+#ifdef __APPLE__
+		int arg_on=1;
+		setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, (char *) &arg_on, sizeof(int));
+		send(fd, buff, 5, 0);
+#else
 		send(fd, buff, 5, MSG_NOSIGNAL);
+#endif
 	}
 //	int rc=0;
 	mysql_close_no_command(mysql);
-/*
-	if (mysql->net.vio) {
-		rc=shutdown(fd, SHUT_RDWR);
-		if (rc) {
-			proxy_error("shutdown(): FD=%d , code=%d\n", fd, errno);
-			assert(rc==0);
-		}
-		close(fd);
-	}
-*/
 }
 
 
@@ -1632,3 +1654,27 @@ int MySQL_Connection::async_send_simple_command(short event, char *stmt, unsigne
 	}
 	return 1;
 }
+
+void MySQL_Connection::reset() {
+	status_flags=0;
+	reusable=true;
+	options.last_set_autocommit=-1; // never sent
+	{ // bug #1160
+		options.sql_mode_int = 0;
+		if (options.sql_mode) {
+			free(options.sql_mode);
+			options.sql_mode = NULL;
+		}
+		options.time_zone_int = 0;
+		if (options.time_zone) {
+			free(options.time_zone);
+			options.time_zone = NULL;
+		}
+	}
+	delete local_stmts;
+#ifndef PROXYSQL_STMT_V14
+	local_stmts=new MySQL_STMTs_local(false);
+#else
+	local_stmts=new MySQL_STMTs_local_v14(false);
+#endif
+}
