@@ -2,6 +2,9 @@
 #include <thread>
 #include "btree_map.h"
 #include "proxysql.h"
+#ifdef __FreeBSD__
+#include <fcntl.h>
+#endif
 
 //#define PROXYSQL_EXTERN
 #include "cpp.h"
@@ -19,6 +22,35 @@
 #endif
 
 
+void parent_open_error_log() {
+	if (GloVars.global.foreground==false) {
+		int outfd=0;
+		int errfd=0;
+		outfd=open(GloVars.errorlog, O_WRONLY | O_APPEND | O_CREAT , S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+		if (outfd>0) {
+			dup2(outfd, STDOUT_FILENO);
+			close(outfd);
+		} else {
+			proxy_error("Impossible to open file\n");
+		}
+		errfd=open(GloVars.errorlog, O_WRONLY | O_APPEND | O_CREAT , S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+		if (errfd>0) {
+			dup2(errfd, STDERR_FILENO);
+			close(errfd);
+		} else {
+			proxy_error("Impossible to open file\n");
+		}
+	}
+}
+
+
+void parent_close_error_log() {
+	if (GloVars.global.foreground==false) {
+		close(STDOUT_FILENO);
+		close(STDERR_FILENO);
+	}
+}
+
 time_t laststart;
 pid_t pid;
 
@@ -131,7 +163,10 @@ static volatile int load_;
 //#ifdef DEBUG
 //const char *malloc_conf = "xmalloc:true,lg_tcache_max:16,purge:decay,junk:true,tcache:false";
 //#else
-const char *malloc_conf = "xmalloc:true,lg_tcache_max:16,purge:decay";
+//const char *malloc_conf = "xmalloc:true,lg_tcache_max:16,purge:decay";
+#ifndef __FreeBSD__
+const char *malloc_conf = "xmalloc:true,lg_tcache_max:16,purge:decay,prof:true,prof_leak:true,lg_prof_sample:20,lg_prof_interval:30,prof_active:false";
+#endif
 //#endif /* DEBUG */
 //const char *malloc_conf = "prof_leak:true,lg_prof_sample:0,prof_final:true,xmalloc:true,lg_tcache_max:16";
 
@@ -141,21 +176,44 @@ int socket_fd;
 
 Query_Cache *GloQC;
 MySQL_Authentication *GloMyAuth;
+#ifdef PROXYSQLCLICKHOUSE
+ClickHouse_Authentication *GloClickHouseAuth;
+#endif /* PROXYSQLCLICKHOUSE */
 Query_Processor *GloQPro;
 ProxySQL_Admin *GloAdmin;
 MySQL_Threads_Handler *GloMTH;
 
+#ifndef PROXYSQL_STMT_V14
 MySQL_STMT_Manager *GloMyStmt;
+#else
+MySQL_STMT_Manager_v14 *GloMyStmt;
+#endif
 
 MySQL_Monitor *GloMyMon;
 std::thread *MyMon_thread;
 
 MySQL_Logger *GloMyLogger;
 
+SQLite3_Server *GloSQLite3Server;
+#ifdef PROXYSQLCLICKHOUSE
+ClickHouse_Server *GloClickHouseServer;
+#endif /* PROXYSQLCLICKHOUSE */
+
+
+ProxySQL_Cluster *GloProxyCluster = NULL;
 
 void * mysql_worker_thread_func(void *arg) {
 
 //	__thr_sfp=l_mem_init();
+
+	pthread_attr_t thread_attr;
+	size_t tmp_stack_size=0;
+	if (!pthread_attr_init(&thread_attr)) {
+		if (!pthread_attr_getstacksize(&thread_attr , &tmp_stack_size )) {
+			__sync_fetch_and_add(&GloVars.statuses.stack_memory_mysql_threads,tmp_stack_size);
+		}
+	}
+
 	proxysql_mysql_thread_t *mysql_thread=(proxysql_mysql_thread_t *)arg;
 	MySQL_Thread *worker = new MySQL_Thread();
 	mysql_thread->worker=worker;
@@ -169,11 +227,21 @@ void * mysql_worker_thread_func(void *arg) {
 	//delete worker;
 	delete worker;
 //	l_mem_destroy(__thr_sfp);
+	__sync_fetch_and_sub(&GloVars.statuses.stack_memory_mysql_threads,tmp_stack_size);
 	return NULL;
 }
 
+#ifdef IDLE_THREADS
 void * mysql_worker_thread_func_idles(void *arg) {
 
+	pthread_attr_t thread_attr;
+	size_t tmp_stack_size=0;
+	if (!pthread_attr_init(&thread_attr)) {
+		if (!pthread_attr_getstacksize(&thread_attr , &tmp_stack_size )) {
+			__sync_fetch_and_add(&GloVars.statuses.stack_memory_mysql_threads,tmp_stack_size);
+		}
+	}
+
 //	__thr_sfp=l_mem_init();
 	proxysql_mysql_thread_t *mysql_thread=(proxysql_mysql_thread_t *)arg;
 	MySQL_Thread *worker = new MySQL_Thread();
@@ -189,8 +257,12 @@ void * mysql_worker_thread_func_idles(void *arg) {
 	//delete worker;
 	delete worker;
 //	l_mem_destroy(__thr_sfp);
+
+	__sync_fetch_and_sub(&GloVars.statuses.stack_memory_mysql_threads,tmp_stack_size);
+
 	return NULL;
 }
+#endif // IDLE_THREADS
 
 void * mysql_shared_query_cache_funct(void *arg) {
 	GloQC->purgeHash_thread(NULL);
@@ -261,20 +333,30 @@ void ProxySQL_Main_init_main_modules() {
 	GloQPro=NULL;
 	GloMTH=NULL;
 	GloMyAuth=NULL;
+#ifdef PROXYSQLCLICKHOUSE
+	GloClickHouseAuth=NULL;
+#endif /* PROXYSQLCLICKHOUSE */
 	GloMyMon=NULL;
 	GloMyLogger=NULL;
 	GloMyStmt=NULL;
 	MyHGM=new MySQL_HostGroups_Manager();
 	GloMTH=new MySQL_Threads_Handler();
 	GloMyLogger = new MySQL_Logger();
+#ifndef PROXYSQL_STMT_V14
 	GloMyStmt=new MySQL_STMT_Manager();
+#else
+	GloMyStmt=new MySQL_STMT_Manager_v14();
+#endif
 }
 
 
 void ProxySQL_Main_init_Admin_module() {
+	// cluster module needs to be initialized before
+	GloProxyCluster = new ProxySQL_Cluster();
+	GloProxyCluster->init();
+	GloProxyCluster->print_version();
 	GloAdmin = new ProxySQL_Admin();
 	GloAdmin->init();
-//	GloAdmin->flush_error_log();
 	GloAdmin->print_version();
 }
 
@@ -293,10 +375,20 @@ void ProxySQL_Main_init_Query_module() {
 void ProxySQL_Main_init_MySQL_Threads_Handler_module() {
 	unsigned int i;
 	GloMTH->init();
-	load_ = GloMTH->num_threads * 2 + 1;
+	load_ = 1;
+	load_ += GloMTH->num_threads;
+#ifdef IDLE_THREADS
+	if (GloVars.global.idle_threads) {
+		load_ += GloMTH->num_threads;
+	}
+#endif // IDLE_THREADS
 	for (i=0; i<GloMTH->num_threads; i++) {
 		GloMTH->create_thread(i,mysql_worker_thread_func, false);
-		GloMTH->create_thread(i,mysql_worker_thread_func_idles, true);
+#ifdef IDLE_THREADS
+		if (GloVars.global.idle_threads) {
+			GloMTH->create_thread(i,mysql_worker_thread_func_idles, true);
+		}
+#endif // IDLE_THREADS
 	}
 }
 
@@ -308,11 +400,32 @@ void ProxySQL_Main_init_Query_Cache_module() {
 
 void ProxySQL_Main_init_MySQL_Monitor_module() {
 	// start MySQL_Monitor
-	GloMyMon = new MySQL_Monitor();
+//	GloMyMon = new MySQL_Monitor();
 	MyMon_thread = new std::thread(&MySQL_Monitor::run,GloMyMon);
 	GloMyMon->print_version();
 }
 
+
+void ProxySQL_Main_init_SQLite3Server() {
+	// start SQLite3Server
+	GloSQLite3Server = new SQLite3_Server();
+	GloSQLite3Server->init();
+	GloAdmin->init_sqliteserver_variables();
+	GloSQLite3Server->print_version();
+}
+#ifdef PROXYSQLCLICKHOUSE
+void ProxySQL_Main_init_ClickHouseServer() {
+	// start SQServer
+	GloClickHouseServer = new ClickHouse_Server();
+	GloClickHouseServer->init();
+	GloAdmin->init_clickhouse_variables();
+	GloClickHouseServer->print_version();
+	GloClickHouseAuth = new ClickHouse_Authentication();
+	GloClickHouseAuth->print_version();
+	GloAdmin->init_clickhouse_users();
+}
+#endif /* PROXYSQLCLICKHOUSE */
+
 void ProxySQL_Main_join_all_threads() {
 	cpu_timer t;
 	if (GloMTH) {
@@ -376,6 +489,32 @@ void ProxySQL_Main_shutdown_all_modules() {
 		GloQPro=NULL;
 #ifdef DEBUG
 		std::cerr << "GloQPro shutdown in ";
+#endif
+	}
+#ifdef PROXYSQLCLICKHOUSE
+	if (GloClickHouseAuth) {
+		cpu_timer t;
+		delete GloClickHouseAuth;
+		GloClickHouseAuth=NULL;
+#ifdef DEBUG
+		std::cerr << "GloClickHouseAuth shutdown in ";
+#endif
+	}
+	if (GloClickHouseServer) {
+		cpu_timer t;
+		delete GloClickHouseServer;
+		GloClickHouseServer=NULL;
+#ifdef DEBUG
+		std::cerr << "GloClickHouseServer shutdown in ";
+#endif
+	}
+#endif /* PROXYSQLCLICKHOUSE */
+	if (GloSQLite3Server) {
+		cpu_timer t;
+		delete GloSQLite3Server;
+		GloSQLite3Server=NULL;
+#ifdef DEBUG
+		std::cerr << "GloSQLite3Server shutdown in ";
 #endif
 	}
 	if (GloMyAuth) {
@@ -453,6 +592,13 @@ void ProxySQL_Main_init_phase2___not_started() {
 	ProxySQL_Main_init_Admin_module();
 	GloMTH->print_version();
 
+	{
+		cpu_timer t;
+		GloMyLogger->set_datadir(GloVars.datadir);
+#ifdef DEBUG
+		std::cerr << "Main phase3 : GloMyLogger initialized in ";
+#endif
+	}
 	if (GloVars.configfile_open) {
 		GloVars.confFile->CloseFile();
 	}
@@ -474,10 +620,13 @@ void ProxySQL_Main_init_phase3___start_all() {
 		std::cerr << "Main phase3 : GloMyLogger initialized in ";
 #endif
 	}
+	// Initialized monitor, no matter if it will be started or not
+	GloMyMon = new MySQL_Monitor();
 	// load all mysql servers to GloHGH
 	{
 		cpu_timer t;
 		GloAdmin->init_mysql_servers();
+		GloAdmin->init_proxysql_servers();
 		GloAdmin->load_scheduler_to_runtime();
 #ifdef DEBUG
 		std::cerr << "Main phase3 : GloAdmin initialized in ";
@@ -523,6 +672,22 @@ void ProxySQL_Main_init_phase3___start_all() {
 			std::cerr << "Main phase3 : MySQL Monitor initialized in ";
 #endif
 		}
+	if ( GloVars.global.sqlite3_server == true ) {
+		cpu_timer t;
+		ProxySQL_Main_init_SQLite3Server();
+#ifdef DEBUG
+		std::cerr << "Main phase3 : SQLite3 Server initialized in ";
+#endif
+	}
+#ifdef PROXYSQLCLICKHOUSE
+	if ( GloVars.global.clickhouse_server == true ) {
+		cpu_timer t;
+		ProxySQL_Main_init_ClickHouseServer();
+#ifdef DEBUG
+		std::cerr << "Main phase3 : ClickHouse Server initialized in ";
+#endif
+	}
+#endif /* PROXYSQLCLICKHOUSE */
 }
 
 
@@ -618,9 +783,12 @@ bool ProxySQL_daemonize_phase3() {
 	int rc;
 	int status;
 	//daemon_log(LOG_INFO, "Angel process started ProxySQL process %d\n", pid);
+	parent_open_error_log();
 	proxy_info("Angel process started ProxySQL process %d\n", pid);
+	parent_close_error_log();
 	rc=waitpid(pid, &status, 0);
 	if (rc==-1) {
+		parent_open_error_log();
 		perror("waitpid");
 		//proxy_error("[FATAL]: waitpid: %s\n", perror("waitpid"));
 		exit(EXIT_FAILURE);
@@ -630,16 +798,21 @@ bool ProxySQL_daemonize_phase3() {
 		rc=WEXITSTATUS(status);
 		if (rc==0) {
 			//daemon_log(LOG_INFO, "Shutdown angel process\n");
+			parent_open_error_log();
 			proxy_info("Shutdown angel process\n");
 			exit(EXIT_SUCCESS);
 		} else {
 			//daemon_log(LOG_INFO, "ProxySQL exited with code %d . Restarting!\n", rc);
+			parent_open_error_log();
 			proxy_error("ProxySQL exited with code %d . Restarting!\n", rc);
+			parent_close_error_log();
 			return false;
 		}
 	} else {
 		//daemon_log(LOG_INFO, "ProxySQL crashed. Restarting!\n");
+		parent_open_error_log();
 		proxy_error("ProxySQL crashed. Restarting!\n");
+		parent_close_error_log();
 		return false;
 	}
 	return true;
@@ -700,26 +873,37 @@ int main(int argc, const char * argv[]) {
 	if (glovars.proxy_restart_on_error) {
 gotofork:
 		if (laststart) {
-			//daemon_log(LOG_INFO, "Angel process is waiting %d seconds before starting a new ProxySQL process\n", glovars.proxy_restart_delay);
-			proxy_info("Angel process is waiting %d seconds before starting a new ProxySQL process\n", glovars.proxy_restart_delay);
-			sleep(glovars.proxy_restart_delay);
+			int currenttime=time(NULL);
+			if (currenttime == laststart) { /// we do not want to restart multiple times in the same second
+				// if restart is too frequent, something really bad is going on
+				//daemon_log(LOG_INFO, "Angel process is waiting %d seconds before starting a new ProxySQL process\n", glovars.proxy_restart_delay);
+				parent_open_error_log();
+				proxy_info("Angel process is waiting %d seconds before starting a new ProxySQL process\n", glovars.proxy_restart_delay);
+				parent_close_error_log();
+				sleep(glovars.proxy_restart_delay);
+			}
 		}
 		laststart=time(NULL);
 		pid = fork();
 		if (pid < 0) {
 			//daemon_log(LOG_INFO, "[FATAL]: Error in fork()\n");
+			parent_open_error_log();
 			proxy_error("[FATAL]: Error in fork()\n");
 			exit(EXIT_FAILURE);
 		}
 
 		if (pid) { /* The parent */
 
+			parent_close_error_log();
 			if (ProxySQL_daemonize_phase3()==false) {
 				goto gotofork;
 			}
 
 		} else { /* The daemon */
 
+			// we open the files also on the child process
+			// this is required if the child process was created after a crash
+			parent_open_error_log();
 			GloVars.global.start_time=monotonic_time();
 			GloVars.install_signal_handler();
 		}
